package ru.dbotthepony.mc.otm.datagen

import net.minecraft.core.Direction
import net.minecraft.data.recipes.ShapedRecipeBuilder
import net.minecraft.data.recipes.ShapelessRecipeBuilder
import net.minecraft.resources.ResourceLocation
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.IronBarsBlock
import net.minecraftforge.client.model.generators.ModelFile
import net.minecraftforge.common.Tags
import net.minecraftforge.common.loot.LootTableIdCondition
import net.minecraftforge.eventbus.api.SubscribeEvent
import net.minecraftforge.fml.common.Mod
import net.minecraftforge.forge.event.lifecycle.GatherDataEvent
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.registry.MBlocks
import ru.dbotthepony.mc.otm.registry.MItems
import ru.dbotthepony.mc.otm.block.DriveViewerBlock
import ru.dbotthepony.mc.otm.block.EnergyCounterBlock
import ru.dbotthepony.mc.otm.block.RotatableMatteryBlock
import ru.dbotthepony.mc.otm.block.PatternStorageBlock
import ru.dbotthepony.mc.otm.block.entity.worker.WorkerState
import ru.dbotthepony.mc.otm.data.LootTableBasicAppender
import ru.dbotthepony.mc.otm.datagen.blocks.BatteryBankProvider
import ru.dbotthepony.mc.otm.datagen.blocks.MatterBankProvider
import ru.dbotthepony.mc.otm.datagen.blocks.MatteryBlockStateProvider
import ru.dbotthepony.mc.otm.datagen.loot.LootModifiers
import ru.dbotthepony.mc.otm.datagen.items.MatteryItemModelProvider
import ru.dbotthepony.mc.otm.datagen.loot.LootTables
import ru.dbotthepony.mc.otm.datagen.loot.TileNbtCopy
import ru.dbotthepony.mc.otm.datagen.models.BlockMatteryModelProvider
import ru.dbotthepony.mc.otm.datagen.recipes.MatteryRecipeProvider
import ru.dbotthepony.mc.otm.datagen.recipes.has
import ru.dbotthepony.mc.otm.registry.MItemTags

@Mod.EventBusSubscriber(modid = DataGen.MOD_ID, bus = Mod.EventBusSubscriber.Bus.MOD)
object DataGen {
	const val MOD_ID = OverdriveThatMatters.MOD_ID

	private lateinit var blockModelProvider: BlockMatteryModelProvider
	private lateinit var itemModelProvider: MatteryItemModelProvider
	private lateinit var blockStateProvider: MatteryBlockStateProvider
	private lateinit var lootTableProvider: LootTables
	private lateinit var recipeProvider: MatteryRecipeProvider
	private lateinit var lootModifier: LootModifiers

	private fun decorativeCubeAll(vararg blocks: Block) {
		blockModelProvider.decorativeCubeAll(*blocks)
		blockStateProvider.simpleBlockM(*blocks)
	}

	private fun decorativeCubeAll(blocks: List<Block>) {
		blockModelProvider.decorativeCubeAll(blocks)
		blockStateProvider.simpleBlockM(blocks)
	}

	private fun resourceCubeAll(vararg blocks: Block) {
		blockModelProvider.resourceCubeAll(*blocks)
		blockStateProvider.simpleBlockM(*blocks)
	}

	private fun decorativeColumn(block: Block, side: String, end: String) {
		blockModelProvider.decorativeColumn(block, side, end)
		blockStateProvider.simpleBlockM(block)
	}

	private fun pane(block: Block, textureSide: ResourceLocation, textureRailing: ResourceLocation) {
		val name = block.registryName?.path ?: throw IllegalStateException("Invalid state of glass pane $block")

		val post = name
		val side = "${name}_ne"
		val side_alt = "${name}_sw"
		val noside = "${name}_noside_ne"
		val noside_alt = "${name}_noside_sw"

		var mdl_post: ModelFile? = null
		var mdl_side: ModelFile? = null
		var mdl_side_alt: ModelFile? = null
		var mdl_noside: ModelFile? = null
		var mdl_noside_alt: ModelFile? = null

		with(blockModelProvider) {
			lambda {
				mdl_post = panePost(post, textureSide, textureRailing)
				mdl_side = paneSide(side, textureSide, textureRailing)
				mdl_side_alt = paneSideAlt(side_alt, textureSide, textureRailing)
				mdl_noside = paneNoSide(noside, textureSide)
				mdl_noside_alt = paneNoSideAlt(noside_alt, textureSide)
			}
		}

		blockStateProvider.lambda {
			check(mdl_post != null) { "DataGen execution order is screwed lmaooooooo" }

			with(it.getMultipartBuilder(block)) {
				part().modelFile(mdl_post!!).addModel()
				part().modelFile(mdl_side!!).addModel().condition(IronBarsBlock.NORTH, true)
				part().modelFile(mdl_side!!).rotationY(90).addModel().condition(IronBarsBlock.EAST, true)
				part().modelFile(mdl_side_alt!!).addModel().condition(IronBarsBlock.SOUTH, true)
				part().modelFile(mdl_side_alt!!).rotationY(90).addModel().condition(IronBarsBlock.WEST, true)
				part().modelFile(mdl_noside!!).addModel().condition(IronBarsBlock.NORTH, false)
				part().modelFile(mdl_noside!!).rotationY(270).addModel().condition(IronBarsBlock.WEST, false)
				part().modelFile(mdl_noside_alt!!).addModel().condition(IronBarsBlock.EAST, false)
				part().modelFile(mdl_noside_alt!!).rotationY(90).addModel().condition(IronBarsBlock.SOUTH, false)
			}
		}
	}

	@SubscribeEvent
	@JvmStatic
	@Suppress("unused")
	fun onGatherData(event: GatherDataEvent) {
		blockModelProvider = BlockMatteryModelProvider(event)
		blockStateProvider = MatteryBlockStateProvider(event)
		itemModelProvider = MatteryItemModelProvider(event)
		lootTableProvider = LootTables(event.generator)
		recipeProvider = MatteryRecipeProvider(event.generator)
		lootModifier = LootModifiers(event.generator)

		event.generator.addProvider(blockModelProvider)
		event.generator.addProvider(itemModelProvider)
		event.generator.addProvider(blockStateProvider)
		event.generator.addProvider(blockStateProvider)
		event.generator.addProvider(recipeProvider)
		event.generator.addProvider(MatterBankProvider(event))
		event.generator.addProvider(BatteryBankProvider(event))
		event.generator.addProvider(lootTableProvider)
		event.generator.addProvider(lootModifier)

		decorativeCubeAll(MBlocks.CRATE_LIST)
		decorativeCubeAll(MBlocks.CARBON_FIBRE_BLOCK)
		decorativeCubeAll(MBlocks.TRITANIUM_BLOCK)

		for (glass in MBlocks.INDUSTRIAL_GLASS_LIST) {
			decorativeCubeAll(glass)
		}

		blockModelProvider.resourceCubeAll(MBlocks.TRITANIUM_ORE)
		blockModelProvider.resourceCubeAll(MBlocks.TRITANIUM_RAW_BLOCK)
		blockModelProvider.resourceCubeAll(MBlocks.DEEPSLATE_TRITANIUM_ORE)

		decorativeColumn(MBlocks.TRITANIUM_STRIPED_BLOCK, "tritanium_striped_block", "tritanium_block")

		for ((i, glass) in MBlocks.INDUSTRIAL_GLASS_PANE_LIST.withIndex()) {
			val name = MBlocks.INDUSTRIAL_GLASS_LIST[i].registryName!!.path
			val textureSide = ResourceLocation(MOD_ID, "block/decorative/$name")
			val textureRailing = textureSide
			pane(glass, textureSide, textureRailing)
		}

		with(blockModelProvider) {
			lambda {
				copy("block/battery/battery1", "block/battery/battery0").also { it.offset(-4f, 0f, 0f) }
				copy("block/battery/battery2", "block/battery/battery0").also { it.offset(-8f, 0f, 0f) }

				copy("block/battery/battery3", "block/battery/battery0").also { it.offset(0f, 6f, 0f) }
				copy("block/battery/battery4", "block/battery/battery0").also { it.offset(-4f, 6f, 0f) }
				copy("block/battery/battery5", "block/battery/battery0").also { it.offset(-8f, 6f, 0f) }

				copy("block/battery/battery7", "block/battery/battery6").also { it.offset(4f, 0f, 0f) }
				copy("block/battery/battery8", "block/battery/battery6").also { it.offset(8f, 0f, 0f) }

				copy("block/battery/battery9", "block/battery/battery6").also { it.offset(0f, 6f, 0f) }
				copy("block/battery/battery10", "block/battery/battery6").also { it.offset(4f, 6f, 0f) }
				copy("block/battery/battery11", "block/battery/battery6").also { it.offset(8f, 6f, 0f) }

				for (i in 0 .. 11) {
					withExistingParent("block/battery/matter_capacitor$i", ResourceLocation(MOD_ID, "block/battery/battery$i"))
						.texture("1", "block/matterybank_core")
				}

				for (i in 1 .. 7) {
					copy("block/pattern/model$i", "block/pattern/model0").also { it.offset(-2f * i, 0f, 0f) }
				}
			}
		}

		with(blockStateProvider) {
			block(MBlocks.BLACK_HOLE)
			block(MBlocks.ANDROID_STATION)

			ore(MBlocks.DEEPSLATE_TRITANIUM_ORE)
			ore(MBlocks.TRITANIUM_ORE)
			ore(MBlocks.TRITANIUM_RAW_BLOCK)

			block(MBlocks.CHEMICAL_GENERATOR)
			block(MBlocks.MATTER_SCANNER)
			block(MBlocks.ITEM_MONITOR)
			block(MBlocks.MATTER_BOTTLER)
			block(MBlocks.MATTER_DECOMPOSER)
			block(MBlocks.MATTER_REPLICATOR)
			block(MBlocks.PLATE_PRESS)
			block(MBlocks.GRAVITATION_STABILIZER)
			block(MBlocks.GRAVITATION_STABILIZER_LENS)
		}

		with(itemModelProvider) {
			block(MItems.ANDROID_STATION)
			block(MItems.BATTERY_BANK)
			block(MItems.MATTER_CAPACITOR_BANK)
			block(MItems.PATTERN_STORAGE)
			block(MItems.BLACK_HOLE)
			block(MItems.CARBON_FIBRE_BLOCK)
			block(MItems.DEEPSLATE_TRITANIUM_ORE)
			block(MItems.TRITANIUM_ORE)
			block(MItems.TRITANIUM_STRIPED_BLOCK)
			block(MItems.TRITANIUM_RAW_BLOCK)
			block(MItems.ITEM_MONITOR)
			block(MItems.TRITANIUM_BLOCK)

			for (glass in MItems.INDUSTRIAL_GLASS_LIST) {
				block(glass)
			}

			for ((i, glass) in MItems.INDUSTRIAL_GLASS_PANE_LIST.withIndex()) {
				generated(glass, ResourceLocation(MOD_ID, "block/decorative/${MItems.INDUSTRIAL_GLASS_LIST[i].registryName!!.path}"))
			}

			blocks(MItems.CRATE_LIST)

			components(MItems.DATAGEN_COMPONENTS)

			generated(MItems.PILL_ANDROID)
			generated(MItems.PILL_HUMANE)
			generated(MItems.PILL_OBLIVION)
			generated(MItems.PILL_HEAL)
			generated(MItems.NUTRIENT_PASTE)
			component(MItems.TRITANIUM_INGOT)
			resource(MItems.TRITANIUM_ORE_CLUMP)

			handheld(MItems.TRITANIUM_TOOLS)
			generated(MItems.TRITANIUM_ARMOR)

			generatedTiered(MItems.BATTERIES, "battery_tier")
			generated(MItems.BATTERY_CREATIVE)

			generated(MItems.MATTER_CAPACITOR_BASIC, ResourceLocation(MOD_ID, "item/matter_capacitor_tier1"))
			generated(MItems.MATTER_CAPACITOR_NORMAL, ResourceLocation(MOD_ID, "item/matter_capacitor_tier2"))
			generated(MItems.MATTER_CAPACITOR_DENSE, ResourceLocation(MOD_ID, "item/matter_capacitor_tier3"))
			generated(MItems.MATTER_CAPACITOR_CREATIVE)

			generated(MItems.PATTERN_DRIVE_CREATIVE)

			block(MItems.CARGO_CRATE, "cargo_crate_closed")
			block(MItems.CHEMICAL_GENERATOR, "chemical_generator_working")
			block(MItems.ENERGY_COUNTER, "energy_counter_down")
			block(MItems.MATTER_BOTTLER, "matter_bottler_working")
			block(MItems.MATTER_CABLE, "matter_cable_core")
			block(MItems.MATTER_DECOMPOSER, "matter_decomposer_working")

			block(MItems.PLATE_PRESS, "plate_press_idle")
		}

		blockStateProvider.lambda { provider ->
			with(provider.getMultipartBuilder(MBlocks.DRIVE_VIEWER)) {
				for (facing in RotatableMatteryBlock.FACING.possibleValues) {
					part()
						.modelFile(provider.models().getExistingFile(ResourceLocation(MOD_ID, "block/drive_viewer_drive_part")))
						.rotationY(facing.toYRotBlockstate())
						.addModel()
						.condition(RotatableMatteryBlock.FACING, facing)
						.condition(DriveViewerBlock.DRIVE_PRESENT, true)

					for (workState in WorkerState.SEMI_WORKER_STATE.possibleValues) {
						part()
							.modelFile(provider.models().getExistingFile(ResourceLocation(MOD_ID, "block/drive_viewer_${workState.name.lowercase()}")))
							.rotationY(facing.toYRotBlockstate())
							.addModel()
							.condition(WorkerState.SEMI_WORKER_STATE, workState)
							.condition(RotatableMatteryBlock.FACING, facing)
					}
				}
			}

			with(provider.getMultipartBuilder(MBlocks.PATTERN_STORAGE)) {
				for (facing in RotatableMatteryBlock.FACING.possibleValues) {
					for (i in 0 .. 7) {
						part()
							.modelFile(provider.models().getExistingFile(ResourceLocation(MOD_ID, "block/pattern/model$i")))
							.rotationY(facing.toYRotBlockstate())
							.addModel()
							.condition(RotatableMatteryBlock.FACING, facing)
							.condition(PatternStorageBlock.PATTERN_STORAGE_DISKS_PROPS[i], true)
					}

					part()
						.modelFile(provider.models().getExistingFile(ResourceLocation(MOD_ID, "block/pattern_storage")))
						.rotationY(facing.toYRotBlockstate())
						.addModel()
						.condition(RotatableMatteryBlock.FACING, facing)
				}
			}

			with(provider.getMultipartBuilder(MBlocks.ENERGY_COUNTER)) {
				// даваааййй
				val up = provider.models().getExistingFile(ResourceLocation(MOD_ID, "block/energy_counter_up"))
				val down = provider.models().getExistingFile(ResourceLocation(MOD_ID, "block/energy_counter_down"))
				val west = provider.models().getExistingFile(ResourceLocation(MOD_ID, "block/energy_counter_west"))
				val east = provider.models().getExistingFile(ResourceLocation(MOD_ID, "block/energy_counter_east"))

				// ДАААА ДАВАЙЙ ДАААВАААЙЙЙЙЙЙ
				val north = provider.models().getExistingFile(ResourceLocation(MOD_ID, "block/energy_counter_north"))
				val north_down = provider.models().getExistingFile(ResourceLocation(MOD_ID, "block/energy_counter_north_down"))
				val north_east = provider.models().getExistingFile(ResourceLocation(MOD_ID, "block/energy_counter_north_east"))
				val north_west = provider.models().getExistingFile(ResourceLocation(MOD_ID, "block/energy_counter_north_west"))
				val south = provider.models().getExistingFile(ResourceLocation(MOD_ID, "block/energy_counter_south"))
				val south_down = provider.models().getExistingFile(ResourceLocation(MOD_ID, "block/energy_counter_south_down"))
				val south_east = provider.models().getExistingFile(ResourceLocation(MOD_ID, "block/energy_counter_south_east"))
				val south_west = provider.models().getExistingFile(ResourceLocation(MOD_ID, "block/energy_counter_south_west"))

				for (dir in arrayOf(Direction.EAST, Direction.WEST, Direction.SOUTH, Direction.NORTH)) {
					part().modelFile(down).rotationY(dir.toYRotBlockstate()).addModel().condition(EnergyCounterBlock.INPUT_DIRECTION, Direction.UP).condition(EnergyCounterBlock.IF_DIRECTION, dir)
					part().modelFile(up).rotationY(dir.toYRotBlockstate()).addModel().condition(EnergyCounterBlock.INPUT_DIRECTION, Direction.DOWN).condition(EnergyCounterBlock.IF_DIRECTION, dir)
				}

				// низкий поклон за полностью рабочий поворот вокруг оси Z
				part().modelFile(north).addModel().condition(EnergyCounterBlock.INPUT_DIRECTION, Direction.NORTH).condition(EnergyCounterBlock.IF_DIRECTION, Direction.UP)
				part().modelFile(north_down).addModel().condition(EnergyCounterBlock.INPUT_DIRECTION, Direction.NORTH).condition(EnergyCounterBlock.IF_DIRECTION, Direction.DOWN)
				part().modelFile(north_east).addModel().condition(EnergyCounterBlock.INPUT_DIRECTION, Direction.NORTH).condition(EnergyCounterBlock.IF_DIRECTION, Direction.EAST)
				part().modelFile(north_west).addModel().condition(EnergyCounterBlock.INPUT_DIRECTION, Direction.NORTH).condition(EnergyCounterBlock.IF_DIRECTION, Direction.WEST)

				part().modelFile(south).addModel().condition(EnergyCounterBlock.INPUT_DIRECTION, Direction.SOUTH).condition(EnergyCounterBlock.IF_DIRECTION, Direction.UP)
				part().modelFile(south_down).addModel().condition(EnergyCounterBlock.INPUT_DIRECTION, Direction.SOUTH).condition(EnergyCounterBlock.IF_DIRECTION, Direction.DOWN)
				part().modelFile(south_east).addModel().condition(EnergyCounterBlock.INPUT_DIRECTION, Direction.SOUTH).condition(EnergyCounterBlock.IF_DIRECTION, Direction.EAST)
				part().modelFile(south_west).addModel().condition(EnergyCounterBlock.INPUT_DIRECTION, Direction.SOUTH).condition(EnergyCounterBlock.IF_DIRECTION, Direction.WEST)

				for (dir in arrayOf(Direction.WEST, Direction.EAST)) {
					val mdl = if (dir === Direction.WEST) west else east

					part().modelFile(mdl).addModel().condition(EnergyCounterBlock.INPUT_DIRECTION, dir).condition(EnergyCounterBlock.IF_DIRECTION, Direction.NORTH)
					part().modelFile(mdl).rotationX(-90).addModel().condition(EnergyCounterBlock.INPUT_DIRECTION, dir).condition(EnergyCounterBlock.IF_DIRECTION, Direction.UP)
					part().modelFile(mdl).rotationX(180).addModel().condition(EnergyCounterBlock.INPUT_DIRECTION, dir).condition(EnergyCounterBlock.IF_DIRECTION, Direction.SOUTH)
					part().modelFile(mdl).rotationX(90).addModel().condition(EnergyCounterBlock.INPUT_DIRECTION, dir).condition(EnergyCounterBlock.IF_DIRECTION, Direction.DOWN)
				}
			}
		}

		val workerTags = arrayOf(
			TileNbtCopy("work_ticks"),
			TileNbtCopy("current_job"),
			TileNbtCopy("battery_container"),
			TileNbtCopy("energy"),
		)

		with(lootTableProvider) {
			simpleBlocks(MBlocks.CRATE_LIST)
			simpleBlock(MBlocks.CARGO_CRATE)
			simpleBlock(MBlocks.CARBON_FIBRE_BLOCK)
			simpleBlock(MBlocks.TRITANIUM_RAW_BLOCK)
			simpleBlock(MBlocks.TRITANIUM_BLOCK)
			simpleBlock(MBlocks.TRITANIUM_STRIPED_BLOCK)
			simpleBlock(MBlocks.MATTER_CABLE)
			simpleBlock(MBlocks.GRAVITATION_STABILIZER)

			tile(MBlocks.ENERGY_COUNTER, TileNbtCopy("passed"), TileNbtCopy("history"), TileNbtCopy("history_tick"))
			tile(MBlocks.CHEMICAL_GENERATOR, TileNbtCopy("energy"), TileNbtCopy("container"), TileNbtCopy("working_ticks"), TileNbtCopy("working_ticks_total"))

			tile(MBlocks.ANDROID_STATION, TileNbtCopy("energy"), TileNbtCopy("battery_container"))
			tile(MBlocks.BATTERY_BANK, TileNbtCopy("container"))
			tile(MBlocks.DRIVE_VIEWER, TileNbtCopy("energy"), TileNbtCopy("container"), TileNbtCopy("battery_container"))

			tile(MBlocks.MATTER_DECOMPOSER, TileNbtCopy("container"), TileNbtCopy("matter"), *workerTags)
			tile(MBlocks.MATTER_REPLICATOR, TileNbtCopy("container"), TileNbtCopy("matter"), *workerTags)
			tile(MBlocks.MATTER_RECYCLER, TileNbtCopy("container"), TileNbtCopy("matter"), *workerTags)
			tile(MBlocks.MATTER_SCANNER, TileNbtCopy("container"), *workerTags)
			tile(MBlocks.PLATE_PRESS, TileNbtCopy("container"), *workerTags)

			tile(MBlocks.MATTER_PANEL, TileNbtCopy("tasks"))
			tile(MBlocks.PATTERN_STORAGE, TileNbtCopy("patterns"))
			tile(MBlocks.MATTER_CAPACITOR_BANK, TileNbtCopy("container"))
			tile(MBlocks.MATTER_BOTTLER, TileNbtCopy("energy"), TileNbtCopy("battery_container"), TileNbtCopy("work_slots"), TileNbtCopy("work_flow"), TileNbtCopy("matter"))
		}

		with(recipeProvider) {
			val baselineMetals = arrayOf("iron", "silver", "bronze", "lead", "constantan")
			val softMetals = arrayOf("gold", "aluminum", "aluminium", "brass", "copper", "electrum")
			val hardMetals = arrayOf("tritanium", "steel", "tungsten", "uranium")

			for (thing in baselineMetals) {
				plate(thing)
			}

			for (thing in softMetals) {
				plate(thing, workTicks = 140)
			}

			for (thing in hardMetals) {
				plate(thing, workTicks = 300)
			}

			lambda { _, consumer ->
				ShapelessRecipeBuilder(MItems.INDUSTRIAL_GLASS, 8)
					.requires(net.minecraft.world.item.Items.GLASS, 8)
					.requires(MItemTags.PLATE_TRITANIUM)
					.unlockedBy("has_plate", has(MItemTags.PLATE_TRITANIUM))
					.unlockedBy("has_glass", has(net.minecraft.world.item.Items.GLASS))
					.save(consumer)

				ShapedRecipeBuilder(MItems.INDUSTRIAL_GLASS_PANE, 16)
					.define('#', MItems.INDUSTRIAL_GLASS)
					.pattern("###").pattern("###")
					.unlockedBy("has_tritanium_glass", has(MItems.INDUSTRIAL_GLASS))
					.save(consumer)

				val mapping = listOf(
					Tags.Items.DYES_WHITE,
					Tags.Items.DYES_ORANGE,
					Tags.Items.DYES_MAGENTA,
					Tags.Items.DYES_LIGHT_BLUE,
					Tags.Items.DYES_YELLOW,
					Tags.Items.DYES_LIME,
					Tags.Items.DYES_PINK,
					Tags.Items.DYES_GRAY,
					Tags.Items.DYES_LIGHT_GRAY,
					Tags.Items.DYES_CYAN,
					Tags.Items.DYES_PURPLE,
					Tags.Items.DYES_BLUE,
					Tags.Items.DYES_BROWN,
					Tags.Items.DYES_GREEN,
					Tags.Items.DYES_RED,
					Tags.Items.DYES_BLACK,
				)

				for ((i, tag) in mapping.withIndex()) {
					val item = MItems.INDUSTRIAL_GLASS_LIST[i + 1]
					val paneItem = MItems.INDUSTRIAL_GLASS_PANE_LIST[i + 1]

					ShapelessRecipeBuilder(item, 8)
						.requires(MItems.INDUSTRIAL_GLASS, 8)
						.requires(tag)
						.unlockedBy("has_tritanium_glass", has(MItems.INDUSTRIAL_GLASS))
						.save(consumer)

					ShapelessRecipeBuilder(paneItem, 8)
						.requires(MItems.INDUSTRIAL_GLASS_PANE, 8)
						.requires(tag)
						.unlockedBy("has_tritanium_glass_pane", has(MItems.INDUSTRIAL_GLASS_PANE))
						.save(consumer)

					ShapedRecipeBuilder(paneItem, 16)
						.define('#', item)
						.pattern("###").pattern("###")
						.unlockedBy("has_colored_tritanium_glass", has(paneItem))
						.save(consumer, ResourceLocation(MOD_ID, "${paneItem.registryName!!.path}_alt"))
				}
			}
		}

		lootModifier.lambda {
			it.add("dungeon_pill", LootTableBasicAppender.Companion, LootTableBasicAppender(
				arrayOf(LootTableIdCondition.Builder(ResourceLocation("chests/simple_dungeon")).build()),
				ItemStack(MItems.PILL_ANDROID, 1) to 0.1,
				ItemStack(MItems.PILL_HEAL, 2) to 0.5,
			))

			it.add("mineshaft_pill", LootTableBasicAppender.Companion, LootTableBasicAppender(
				arrayOf(LootTableIdCondition.Builder(ResourceLocation("chests/abandoned_mineshaft")).build()),
				ItemStack(MItems.PILL_ANDROID, 1) to 0.1,
				ItemStack(MItems.PILL_HEAL, 1) to 0.25,
			))

			it.add("desert_pyramid_pill", LootTableBasicAppender.Companion, LootTableBasicAppender(
				arrayOf(LootTableIdCondition.Builder(ResourceLocation("chests/abandoned_mineshaft")).build()),
				ItemStack(MItems.PILL_ANDROID, 1) to 0.05,
				ItemStack(MItems.PILL_HEAL, 1) to 0.3,
			))

			it.add("jungle_temple_pill", LootTableBasicAppender.Companion, LootTableBasicAppender(
				arrayOf(LootTableIdCondition.Builder(ResourceLocation("chests/jungle_temple")).build()),
				ItemStack(MItems.PILL_ANDROID, 1) to 0.1
			))

			it.add("end_city_pill", LootTableBasicAppender.Companion, LootTableBasicAppender(
				arrayOf(LootTableIdCondition.Builder(ResourceLocation("chests/end_city_treasure")).build()),
				ItemStack(MItems.PILL_ANDROID, 1) to 0.1,
				ItemStack(MItems.PILL_HUMANE, 1) to 0.2,
				ItemStack(MItems.PILL_OBLIVION, 1) to 0.4,
			))

			it.add("shipwreck_supply_pill", LootTableBasicAppender.Companion, LootTableBasicAppender(
				arrayOf(LootTableIdCondition.Builder(ResourceLocation("chests/shipwreck_supply")).build()),
				ItemStack(MItems.PILL_HUMANE, 1) to 0.3
			))
		}
	}
}
