package ru.dbotthepony.mc.otm.block;

import com.google.common.collect.ImmutableMap;
import net.minecraft.MethodsReturnNonnullByDefault;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.EntityBlock;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.state.BlockBehaviour;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.level.block.state.properties.BooleanProperty;
import net.minecraft.world.level.material.Material;
import net.minecraft.world.level.material.MaterialColor;
import net.minecraft.world.phys.shapes.BooleanOp;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.world.phys.shapes.Shapes;
import net.minecraft.world.phys.shapes.VoxelShape;
import ru.dbotthepony.mc.otm.block.entity.MatterCableBlockEntity;

import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import java.util.ArrayList;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class BlockMatterCable extends Block implements EntityBlock {
	@Nullable
	@Override
	public BlockEntity newBlockEntity(BlockPos blockPos, BlockState blockState) {
		return new MatterCableBlockEntity(blockPos, blockState);
	}

	public static final BooleanProperty CONNECTION_SOUTH = BooleanProperty.create("connect_south");
	public static final BooleanProperty CONNECTION_WEST = BooleanProperty.create("connect_west");
	public static final BooleanProperty CONNECTION_EAST = BooleanProperty.create("connect_east");
	public static final BooleanProperty CONNECTION_NORTH = BooleanProperty.create("connect_north");
	public static final BooleanProperty CONNECTION_UP = BooleanProperty.create("connect_up");
	public static final BooleanProperty CONNECTION_DOWN = BooleanProperty.create("connect_down");

	public static final BooleanProperty[] MAPPING_CONNECTION_PROP = new BooleanProperty[] {
		CONNECTION_DOWN,
		CONNECTION_UP,
		CONNECTION_NORTH,
		CONNECTION_SOUTH,
		CONNECTION_WEST,
		CONNECTION_EAST
	};

	protected ImmutableMap<BlockState, VoxelShape> SHAPES;

	public BlockMatterCable() {
		super(BlockBehaviour.Properties.of(Material.STONE, MaterialColor.METAL).requiresCorrectToolForDrops().strength(1.0F, 6.0F));
		registerDefaultState(
			this.defaultBlockState()
				.setValue(CONNECTION_DOWN, false)
				.setValue(CONNECTION_UP, false)
				.setValue(CONNECTION_NORTH, false)
				.setValue(CONNECTION_SOUTH, false)
				.setValue(CONNECTION_WEST, false)
				.setValue(CONNECTION_EAST, false)
		);

		SHAPES = getShapeForEachState((state) -> {
			ArrayList<VoxelShape> shapes = new ArrayList<>();

			if (state.getValue(CONNECTION_SOUTH)) {
				shapes.add(Shapes.box(
					0.5 - 0.15,
					0.5 - 0.15,
					0.65,

					0.5 + 0.15,
					0.5 + 0.15,
					1
				));
			}

			if (state.getValue(CONNECTION_NORTH)) {
				shapes.add(Shapes.box(
					0.5 - 0.15,
					0.5 - 0.15,
					0,

					0.5 + 0.15,
					0.5 + 0.15,
					0.35
				));
			}

			if (state.getValue(CONNECTION_DOWN)) {
				shapes.add(Shapes.box(
					0.5 - 0.15,
					0,
					0.5 - 0.15,

					0.5 + 0.15,
					0.5 - 0.15,
					0.5 + 0.15
				));
			}

			if (state.getValue(CONNECTION_UP)) {
				shapes.add(Shapes.box(
					0.5 - 0.15,
					0.5 - 0.15,
					0.5 - 0.15,

					0.5 + 0.15,
					1,
					0.5 + 0.15
				));
			}

			if (state.getValue(CONNECTION_EAST)) {
				shapes.add(Shapes.box(
					0.65,
					0.5 - 0.15,
					0.5 - 0.15,

					1,
					0.5 + 0.15,
					0.5 + 0.15
				));
			}

			if (state.getValue(CONNECTION_WEST)) {
				shapes.add(Shapes.box(
					0,
					0.5 - 0.15,
					0.5 - 0.15,

					0.35,
					0.5 + 0.15,
					0.5 + 0.15
				));
			}

			VoxelShape final_shape = CORE_SHAPE;

			for (VoxelShape add_shape : shapes) {
				final_shape = Shapes.joinUnoptimized(final_shape, add_shape, BooleanOp.OR);
			}

			return final_shape;
		});
	}

	private final VoxelShape CORE_SHAPE = Shapes.box(
		0.5 - 0.15,
		0.5 - 0.15,
		0.5 - 0.15,

		0.5 + 0.15,
		0.5 + 0.15,
		0.5 + 0.15
	);

	@Override
	@SuppressWarnings("deprecation")
	public VoxelShape getShape(BlockState p_60555_, BlockGetter p_60556_, BlockPos p_60557_, CollisionContext p_60558_) {
		VoxelShape get = SHAPES.get(p_60555_);
		return get != null ? get : CORE_SHAPE;
	}

	@Override
	public boolean hasDynamicShape() {
		return true;
	}

	@Override
	protected void createBlockStateDefinition(StateDefinition.Builder<Block, BlockState> builder) {
		builder.add(CONNECTION_SOUTH,
			CONNECTION_WEST,
			CONNECTION_EAST,
			CONNECTION_NORTH,
			CONNECTION_UP,
			CONNECTION_DOWN);
	}
}
