package ru.dbotthepony.mc.otm.capability.matter;

import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.Tag;
import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.ItemStack;
import net.minecraftforge.registries.ForgeRegistries;
import net.minecraftforge.registries.ForgeRegistry;
import net.minecraftforge.registries.RegistryManager;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Objects;
import java.util.UUID;

public record PatternState(@Nonnull UUID id, @Nonnull Item item, double research) {
	public PatternState(@Nonnull UUID id, @Nonnull Item item, double research) {
		this.id = id;
		this.item = item;
		this.research = Math.max(0, Math.min(1, research));
	}

	@Override
	public boolean equals(Object state) {
		if (state instanceof PatternState state1)
			return state1.id.equals(id);

		return false;
	}

	public ItemStack stack() {
		return new ItemStack(item, 1);
	}

	@Override
	public int hashCode() {
		return id.hashCode();
	}

	public CompoundTag serializeNBT() {
		var tag = new CompoundTag();

		tag.putLong("id_m", id.getMostSignificantBits());
		tag.putLong("id_l", id.getLeastSignificantBits());
		tag.putString("item", Objects.requireNonNull(item.getRegistryName()).toString());
		tag.putDouble("research_percent", research);

		return tag;
	}

	@Nullable
	public static PatternState deserializeNBT(Tag nbt) {
		if (nbt instanceof CompoundTag tag) {
			var item = ForgeRegistries.ITEMS.getValue(new ResourceLocation(tag.getString("item")));

			if (item == null)
				return null;

			var id = new UUID(tag.getLong("id_m"), tag.getLong("id_l"));
			var research_percent = tag.getDouble("research_percent");

			return new PatternState(id, item, research_percent);
		}

		return null;
	}

	public void write(FriendlyByteBuf buffer) {
		buffer.writeLong(id.getMostSignificantBits());
		buffer.writeLong(id.getLeastSignificantBits());

		buffer.writeInt(((ForgeRegistry<Item>) ForgeRegistries.ITEMS).getID(item));

		buffer.writeDouble(research);
	}

	@Nullable
	public static PatternState read(FriendlyByteBuf buffer) {
		long ida = buffer.readLong();
		long idb = buffer.readLong();

		int item = buffer.readInt();
		double percent = buffer.readDouble();

		Item get_item = ((ForgeRegistry<Item>) ForgeRegistries.ITEMS).getValue(item);

		if (get_item == null)
			return null;

		return new PatternState(new UUID(ida, idb), get_item, percent);
	}
}
