package ru.dbotthepony.mc.otm.network.android;

import net.minecraft.client.Minecraft;
import net.minecraft.network.FriendlyByteBuf;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.fml.DistExecutor;
import net.minecraftforge.network.NetworkEvent;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.capability.android.AndroidCapabilityPlayer;
import ru.dbotthepony.mc.otm.android.AndroidFeature;
import ru.dbotthepony.mc.otm.android.AndroidFeatureType;
import ru.dbotthepony.mc.otm.registry.MRegistry;

import java.util.Objects;
import java.util.function.Supplier;

public record AndroidFeaturePacket(boolean is_added, AndroidFeatureType<?> feature) {
	public AndroidFeaturePacket(boolean is_added, AndroidFeature feature) {
		this(is_added, feature.getType());
	}

	public void write(FriendlyByteBuf buffer) {
		buffer.writeBoolean(is_added);
		buffer.writeInt(MRegistry.INSTANCE.getANDROID_FEATURES().getID(feature));
	}

	public void play(Supplier<NetworkEvent.Context> context) {
		context.get().setPacketHandled(true);
		context.get().enqueueWork(() -> {
			DistExecutor.unsafeRunWhenOn(Dist.CLIENT, () -> this::playClient);
		});
	}

	public void playClient() {
		var ply = Minecraft.getInstance().player;

		if (ply != null) {
			ply.getCapability(MatteryCapability.ANDROID).ifPresent(cap -> {
				if (cap instanceof AndroidCapabilityPlayer pcap) {
					pcap.addFeature(feature);
				}
			});
		}
	}

	public static AndroidFeaturePacket read(FriendlyByteBuf buffer) {
		var is_added = buffer.readBoolean();
		var feature = Objects.requireNonNull(MRegistry.INSTANCE.getANDROID_FEATURES().getValue(buffer.readInt()));

		return new AndroidFeaturePacket(is_added, feature);
	}
}
