package ru.dbotthepony.mc.otm.network.android;

import net.minecraft.client.Minecraft;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.network.FriendlyByteBuf;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.fml.DistExecutor;
import net.minecraftforge.network.NetworkEvent;
import ru.dbotthepony.mc.otm.android.AndroidResearch;
import ru.dbotthepony.mc.otm.android.AndroidResearchType;
import ru.dbotthepony.mc.otm.capability.android.AndroidCapabilityPlayer;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.registry.MRegistry;

import java.util.function.Supplier;

public record AndroidResearchPacket(AndroidResearchType<?> type, CompoundTag payload) {
	public AndroidResearchPacket(AndroidResearch research) {
		this(research.type, research.serializeNBT());
	}

	public void write(FriendlyByteBuf buffer) {
		buffer.writeInt(MRegistry.INSTANCE.getANDROID_RESEARCH().getID(type));
		buffer.writeNbt(payload);
	}

	public static AndroidResearchPacket read(FriendlyByteBuf buffer) {
		return new AndroidResearchPacket(MRegistry.INSTANCE.getANDROID_RESEARCH().getValue(buffer.readInt()), buffer.readNbt());
	}

	public void play(Supplier<NetworkEvent.Context> context) {
		context.get().setPacketHandled(true);
		context.get().enqueueWork(() -> {
			DistExecutor.unsafeRunWhenOn(Dist.CLIENT, () -> this::playClient);
		});
	}

	public void playClient() {
		var ply = Minecraft.getInstance().player;

		if (ply != null) {
			ply.getCapability(MatteryCapability.ANDROID).ifPresent(cap -> {
				if (cap instanceof AndroidCapabilityPlayer pcap) {
					pcap.getResearch(type).deserializeNBT(payload);
				}
			});
		}
	}
}
