
@file:Suppress("unused")

package ru.dbotthepony.mc.otm

import com.google.gson.JsonArray
import com.google.gson.JsonElement
import com.google.gson.JsonObject
import net.minecraft.client.Minecraft
import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.core.Vec3i
import net.minecraft.nbt.CompoundTag
import net.minecraft.nbt.ListTag
import net.minecraft.nbt.LongArrayTag
import net.minecraft.nbt.Tag
import net.minecraft.world.Container
import net.minecraft.world.entity.Entity
import net.minecraft.world.entity.LivingEntity
import net.minecraft.world.item.ItemStack
import net.minecraft.world.phys.Vec3
import net.minecraftforge.common.util.LazyOptional
import net.minecraftforge.energy.CapabilityEnergy
import net.minecraftforge.items.IItemHandler
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import java.util.*
import java.util.function.Consumer
import kotlin.properties.ReadWriteProperty
import kotlin.reflect.KProperty

operator fun Direction.unaryMinus(): Direction = this.opposite
operator fun Vec3i.unaryMinus(): Vec3i = Vec3i(-x, -y, -z)
operator fun BlockPos.unaryMinus(): BlockPos = BlockPos(-x, -y, -z)

operator fun CompoundTag.set(index: String, value: Tag) = 				put(index, value)
operator fun CompoundTag.set(index: String, value: Int) = 				putInt(index, value)
operator fun CompoundTag.set(index: String, value: Byte) = 				putByte(index, value)
operator fun CompoundTag.set(index: String, value: Short) = 			putShort(index, value)
operator fun CompoundTag.set(index: String, value: Long) = 				putLong(index, value)
operator fun CompoundTag.set(index: String, value: Float) = 			putFloat(index, value)
operator fun CompoundTag.set(index: String, value: Double) = 			putDouble(index, value)
operator fun CompoundTag.set(index: String, value: String) = 			putString(index, value)
operator fun CompoundTag.set(index: String, value: Boolean) = 			putBoolean(index, value)
operator fun CompoundTag.set(index: String, value: ByteArray) = 		putByteArray(index, value)
operator fun CompoundTag.set(index: String, value: IntArray) = 			putIntArray(index, value)
operator fun CompoundTag.set(index: String, value: LongArray) = 		putLongArray(index, value)

@JvmInline
value class CompoundTagInt(val tag: CompoundTag) : ReadWriteProperty<Any, Int> {
	override fun getValue(thisRef: Any, property: KProperty<*>) = tag.getInt(property.name)
	override fun setValue(thisRef: Any, property: KProperty<*>, value: Int) = tag.putInt(property.name, value)
}

@JvmInline
value class CompoundTagLong(val tag: CompoundTag) : ReadWriteProperty<Any, Long> {
	override fun getValue(thisRef: Any, property: KProperty<*>) = tag.getLong(property.name)
	override fun setValue(thisRef: Any, property: KProperty<*>, value: Long) = tag.putLong(property.name, value)
}

@JvmInline
value class CompoundTagByte(val tag: CompoundTag) : ReadWriteProperty<Any, Byte> {
	override fun getValue(thisRef: Any, property: KProperty<*>) = tag.getByte(property.name)
	override fun setValue(thisRef: Any, property: KProperty<*>, value: Byte) = tag.putByte(property.name, value)
}

@JvmInline
value class CompoundTagShort(val tag: CompoundTag) : ReadWriteProperty<Any, Short> {
	override fun getValue(thisRef: Any, property: KProperty<*>) = tag.getShort(property.name)
	override fun setValue(thisRef: Any, property: KProperty<*>, value: Short) = tag.putShort(property.name, value)
}

@JvmInline
value class CompoundTagFloat(val tag: CompoundTag) : ReadWriteProperty<Any, Float> {
	override fun getValue(thisRef: Any, property: KProperty<*>) = tag.getFloat(property.name)
	override fun setValue(thisRef: Any, property: KProperty<*>, value: Float) = tag.putFloat(property.name, value)
}

@JvmInline
value class CompoundTagBoolean(val tag: CompoundTag) : ReadWriteProperty<Any, Boolean> {
	override fun getValue(thisRef: Any, property: KProperty<*>) = tag.getBoolean(property.name)
	override fun setValue(thisRef: Any, property: KProperty<*>, value: Boolean) = tag.putBoolean(property.name, value)
}

@JvmInline
value class CompoundTagDouble(val tag: CompoundTag) : ReadWriteProperty<Any, Double> {
	override fun getValue(thisRef: Any, property: KProperty<*>) = tag.getDouble(property.name)
	override fun setValue(thisRef: Any, property: KProperty<*>, value: Double) = tag.putDouble(property.name, value)
}

@JvmInline
value class CompoundTagString(val tag: CompoundTag) : ReadWriteProperty<Any, String> {
	override fun getValue(thisRef: Any, property: KProperty<*>): String = tag.getString(property.name)
	override fun setValue(thisRef: Any, property: KProperty<*>, value: String) = tag.putString(property.name, value)
}

val EMPTY_UUID = UUID(0L, 0L)

@JvmInline
value class CompoundTagUUID(val tag: CompoundTag) : ReadWriteProperty<Any, UUID> {
	override fun getValue(thisRef: Any, property: KProperty<*>) = (tag.get(property.name) as LongArrayTag?)?.asLongArray?.let { UUID(it[0], it[1]) } ?: EMPTY_UUID
	override fun setValue(thisRef: Any, property: KProperty<*>, value: UUID) = tag.putLongArray(property.name, longArrayOf(value.mostSignificantBits, value.leastSignificantBits))
}

val CompoundTag.ints get() = CompoundTagInt(this)
val CompoundTag.longs get() = CompoundTagLong(this)
val CompoundTag.bytes get() = CompoundTagByte(this)
val CompoundTag.shorts get() = CompoundTagShort(this)
val CompoundTag.floats get() = CompoundTagFloat(this)
val CompoundTag.doubles get() = CompoundTagDouble(this)
val CompoundTag.booleans get() = CompoundTagBoolean(this)
val CompoundTag.strings get() = CompoundTagString(this)
val CompoundTag.uuids get() = CompoundTagUUID(this)

operator fun Container.set(index: Int, value: ItemStack) = setItem(index, value)
operator fun Container.get(index: Int): ItemStack = getItem(index)
operator fun IItemHandler.get(index: Int): ItemStack = getStackInSlot(index)

operator fun JsonObject.set(s: String, value: JsonElement) = add(s, value)

inline fun CompoundTag.ifHas(s: String, consumer: (Tag) -> Unit) {
	val tag = get(s)

	if (tag != null) {
		consumer(tag)
	}
}

inline fun CompoundTag.ifHas(s: String, type: Byte, consumer: (Tag) -> Unit) {
	val tag = get(s)

	if (tag != null && tag.id == type) {
		consumer(tag)
	}
}

inline fun <reified T : Tag> CompoundTag.ifHas(s: String, type: Class<T>, consumer: (T) -> Unit) {
	val tag = get(s)

	if (tag != null && tag::class.java === type) {
		consumer(tag as T)
	}
}

fun <T> LazyOptional<T>.orNull(): T? {
	if (!isPresent) {
		return null
	}

	return resolve().orElse(null)
}

fun <T> LazyOptional<T>.orThrow(): T {
	if (!isPresent) {
		throw IllegalStateException("Capability was expected to be not null")
	}

	return resolve().orElse(null) ?: throw IllegalStateException("Capability was expected to be not null")
}

inline fun <T> LazyOptional<T>.ifPresentK(lambda: (T) -> Unit) {
	if (isPresent) {
		val value = resolve().orElse(null) ?: throw IllegalStateException("Capability was expected to be not null")
		lambda.invoke(value)
	}
}

val ItemStack.tagNotNull: CompoundTag get() = orCreateTag

inline var Entity.position: Vec3
	get() = position()
	set(value) { setPos(value) }
