package ru.dbotthepony.mc.otm.android

import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.network.chat.TranslatableComponent
import net.minecraftforge.common.util.INBTSerializable
import net.minecraftforge.event.entity.living.LivingHurtEvent
import net.minecraftforge.registries.ForgeRegistryEntry
import ru.dbotthepony.mc.otm.capability.android.AndroidCapability
import ru.dbotthepony.mc.otm.set

open class AndroidFeatureType<T : AndroidFeature> : ForgeRegistryEntry<AndroidFeatureType<*>> {
	private val factory: (AndroidFeatureType<T>, AndroidCapability) -> T

	constructor(factory: (AndroidCapability) -> T) : super() {
		this.factory = { _, capability -> factory.invoke(capability) }
	}

	constructor(factory: (AndroidFeatureType<T>, AndroidCapability) -> T) : super() {
		this.factory = factory
	}

	fun create(android: AndroidCapability): T {
		return factory.invoke(this, android)
	}

	open fun isApplicable(android: AndroidCapability) = true

	open val displayName: Component get() = registryName?.let { TranslatableComponent("android_feature.${it.namespace}.${it.path}") } ?: TranslatableComponent("android_feature.null.null")
}

abstract class AndroidFeature(val type: AndroidFeatureType<*>, val android: AndroidCapability) : INBTSerializable<CompoundTag> {
	val entity get() = android.entity

	open var level = 0
		set(value) {
			if (value != field) {
				field = value
				applyModifiers()
			}
		}

	open fun tickClient() {}
	open fun tickServer() {}

	/**
	 * Called when it is required to network everything again
	 */
	open fun invalidateNetwork() {}

	open fun applyModifiers() {}
	open fun removeModifiers() {}

	open fun onHurt(event: LivingHurtEvent) {}

	override fun serializeNBT(): CompoundTag {
		return CompoundTag().also {
			it["level"] = level
		}
	}

	override fun deserializeNBT(nbt: CompoundTag) {
		level = nbt.getInt("level")
	}
}

class DummyAndroidFeature(type: AndroidFeatureType<*>, android: AndroidCapability) : AndroidFeature(type, android) {
	override fun serializeNBT(): CompoundTag {
		return CompoundTag()
	}

	override fun deserializeNBT(nbt: CompoundTag) {

	}
}

