package ru.dbotthepony.mc.otm.android.feature

import net.minecraft.nbt.CompoundTag
import net.minecraft.server.level.ServerPlayer
import net.minecraftforge.event.entity.living.LivingHurtEvent
import ru.dbotthepony.mc.otm.android.AndroidFeature
import ru.dbotthepony.mc.otm.capability.android.AndroidCapability
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.registry.AndroidFeatures
import ru.dbotthepony.mc.otm.registry.StatNames
import ru.dbotthepony.mc.otm.set
import kotlin.math.roundToInt

class NanobotsRegeneration(android: AndroidCapability) : AndroidFeature(AndroidFeatures.NANOBOTS_REGENERATION, android) {
	private var ticksPassed = 0
	private var healTicks = 0

	override fun tickServer() {
		if (entity.health > 0f && entity.health < entity.maxHealth) {
			ticksPassed++

			val waitTime = TICKS_BETWEEN_HEAL.getOrElse(healTicks) { TICKS_BETWEEN_HEAL.last() }

			if (ticksPassed > waitTime) {
				val missingHealth = entity.maxHealth - entity.health
				val power = ENERGY_PER_HITPOINT * missingHealth
				val extracted = android.extractEnergyInner(power, false)

				if (extracted.isPositive) {
					healTicks = (healTicks + 1).coerceAtMost(level)
					val healed = (extracted / ENERGY_PER_HITPOINT).toFloat()
					entity.heal(healed)
					(entity as ServerPlayer?)?.awardStat(StatNames.HEALTH_REGENERATED, (healed * 10f).roundToInt())
					ticksPassed = 0
				}
			}
		} else {
			ticksPassed = 0
			healTicks = 0
		}
	}

	override fun onHurt(event: LivingHurtEvent) {
		ticksPassed = 0
		healTicks = 0
	}

	override fun serializeNBT(): CompoundTag {
		return super.serializeNBT().also {
			it["ticksPassed"] = ticksPassed
			it["healTicks"] = healTicks
		}
	}

	override fun deserializeNBT(nbt: CompoundTag) {
		super.deserializeNBT(nbt)
		ticksPassed = nbt.getInt("ticksPassed")
		healTicks = nbt.getInt("healTicks")
	}

	companion object {
		private val ENERGY_PER_HITPOINT = ImpreciseFraction(800)

		private val TICKS_BETWEEN_HEAL = listOf(
			100, // 5 seconds
			80, // 4 seconds
			60, // 3 seconds
			40, // 2 seconds
		)
	}
}
