package ru.dbotthepony.mc.otm.block

import net.minecraft.MethodsReturnNonnullByDefault
import javax.annotation.ParametersAreNonnullByDefault
import net.minecraft.world.level.block.EntityBlock
import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.level.block.entity.BlockEntity
import ru.dbotthepony.mc.otm.block.entity.MatterDecomposerBlockEntity
import net.minecraft.world.level.block.entity.BlockEntityType
import net.minecraft.world.level.block.entity.BlockEntityTicker
import net.minecraft.world.level.block.state.StateDefinition
import net.minecraft.world.level.BlockGetter
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.Block
import net.minecraft.world.phys.shapes.CollisionContext
import net.minecraft.world.phys.shapes.VoxelShape
import ru.dbotthepony.mc.otm.block.entity.worker.WorkerState
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import ru.dbotthepony.mc.otm.shapes.BlockShapes

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
class MatterDecomposerBlock : RotatableMatteryBlock(), EntityBlock {
	override fun newBlockEntity(blockPos: BlockPos, blockState: BlockState): BlockEntity {
		return MatterDecomposerBlockEntity(blockPos, blockState)
	}

	override fun <T : BlockEntity?> getTicker(
		p_153212_: Level,
		p_153213_: BlockState,
		p_153214_: BlockEntityType<T>
	): BlockEntityTicker<T>? {
		if (p_153212_.isClientSide || p_153214_ !== MBlockEntities.MATTER_DECOMPOSER)
			return null

		return BlockEntityTicker {_, _, _, t -> if (t is MatterDecomposerBlockEntity) t.tick()}
	}

	override fun createBlockStateDefinition(builder: StateDefinition.Builder<Block, BlockState>) {
		super.createBlockStateDefinition(builder)
		builder.add(WorkerState.WORKER_STATE)
	}

	override fun getShape(
		p_60555_: BlockState,
		p_60556_: BlockGetter,
		p_60557_: BlockPos,
		p_60558_: CollisionContext
	): VoxelShape {
		return SHAPES[p_60555_.getValue(FACING).ordinal]
	}

	companion object {
		private val def = BlockShapes.MATTER_DECOMPOSER.computeShape()

		private val SHAPES: List<VoxelShape> = listOf(
			def,
			def,
			def,
			BlockShapes.MATTER_DECOMPOSER.rotate(Direction.NORTH).computeShape(),
			BlockShapes.MATTER_DECOMPOSER.rotate(Direction.WEST).computeShape(),
			BlockShapes.MATTER_DECOMPOSER.rotate(Direction.EAST).computeShape()
		)
	}
}