package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.MethodsReturnNonnullByDefault
import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.network.chat.TranslatableComponent
import net.minecraft.server.level.ServerLevel
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.state.BlockState
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.util.LazyOptional
import net.minecraftforge.energy.CapabilityEnergy
import net.minecraftforge.energy.IEnergyStorage
import net.minecraftforge.items.CapabilityItemHandler
import ru.dbotthepony.mc.otm.*
import ru.dbotthepony.mc.otm.block.BatteryBankBlock
import ru.dbotthepony.mc.otm.block.RotatableMatteryBlock
import ru.dbotthepony.mc.otm.capability.*
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.menu.BatteryBankMenu
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import javax.annotation.ParametersAreNonnullByDefault

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
class BatteryBankBlockEntity(p_155229_: BlockPos, p_155230_: BlockState) : MatteryBlockEntity(MBlockEntities.BATTERY_BANK, p_155229_, p_155230_) {
	// 6 на 2
	val container: MatteryContainer = object : MatteryContainer(this::setChanged, CAPACITY) {
		override fun setChanged(slot: Int, new: ItemStack, old: ItemStack) {
			super.setChanged(slot, new, old)

			val level = level
			if (level != null) {
				var state = blockState

				for (i in BatteryBankBlock.BATTERY_SLOTS_PROPS.indices) {
					state = state.setValue(BatteryBankBlock.BATTERY_SLOTS_PROPS[i], getItem(i).getCapability(CapabilityEnergy.ENERGY).isPresent)
				}

				if (state !== blockState) {
					level.setBlock(blockPos, state, Block.UPDATE_CLIENTS)
				}
			}
		}
	}

	private val itemHandler = container.handler(
		{ slot: Int, stack: ItemStack -> stack.getCapability(CapabilityEnergy.ENERGY).isPresent },
		{ slot: Int, amount: Int, stack: ItemStack? -> true }
	)

	private data class BatteryBankDistribution(val distribution: Array<ImpreciseFraction>, val maxThroughput: ImpreciseFraction)
	private enum class BankMode { RECEIVE, EXTRACT, BIDIRECTIONAL }

	private inner class BatteryBankEnergy(private val mode: BankMode) : IMatteryEnergyStorage {
		override fun canExtract() = mode != BankMode.RECEIVE
		override fun canReceive() = mode != BankMode.EXTRACT

		override fun extractEnergyOuter(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
			if (mode == BankMode.RECEIVE)
				return ImpreciseFraction.ZERO

			return extractEnergyInner(howMuch, simulate)
		}

		/**
		 * [mode] = true на приём, false на отдачу
		 */
		fun getDistribution(mode: Boolean): BatteryBankDistribution {
			val distribution = Array(container.containerSize) { ImpreciseFraction.ZERO }
			var summ = ImpreciseFraction.ZERO

			for (i in 0 until container.containerSize) {
				val stack = container.getItem(i)

				if (!stack.isEmpty) {
					stack.energy?.let {
						val diff: ImpreciseFraction

						if (mode) {
							diff = it.receiveEnergy(ImpreciseFraction.LONG_MAX_VALUE, true)
						} else {
							diff = it.extractEnergy(ImpreciseFraction.LONG_MAX_VALUE, true)
						}

						distribution[i] = diff
						summ += distribution[i]
					}
				}
			}

			if (!summ.isZero) {
				for (i in 0 until container.containerSize) {
					distribution[i] = distribution[i] / summ
				}
			}

			return BatteryBankDistribution(distribution, summ)
		}

		private fun distributeEnergy(mode: Boolean, howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
			val distribution = getDistribution(mode)

			if (distribution.maxThroughput.isZero)
				return ImpreciseFraction.ZERO

			val distList = distribution.distribution
			var summ = ImpreciseFraction.ZERO

			for (i in 0 until container.containerSize) {
				if (!distList[i].isZero) {
					val stack = container.getItem(i)

					if (!stack.isEmpty) {
						stack.energy?.let {
							val diff: ImpreciseFraction

							if (mode) {
								diff = it.receiveEnergy(howMuch * distList[i], simulate)
							} else {
								diff = it.extractEnergy(howMuch * distList[i], simulate)
							}

							summ += diff
						}
					}
				}
			}

			if (!simulate && !summ.isZero) {
				setChangedLight()
			}

			return summ
		}

		override fun extractEnergyInner(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
			return distributeEnergy(false, howMuch, simulate)
		}

		override fun receiveEnergyOuter(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
			if (mode == BankMode.EXTRACT)
				return ImpreciseFraction.ZERO

			return receiveEnergyInner(howMuch, simulate)
		}

		override fun receiveEnergyInner(howMuch: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
			return distributeEnergy(true, howMuch, simulate)
		}

		override val batteryLevel: ImpreciseFraction get() {
			var result = ImpreciseFraction.ZERO

			for (i in 0 until container.containerSize) {
				val stack = container.getItem(i)

				if (!stack.isEmpty) {
					stack.energy?.let {
						result += it.energyStoredMattery
					}
				}
			}

			return result
		}

		override val maxBatteryLevel: ImpreciseFraction get() {
			var result = ImpreciseFraction.ZERO

			for (i in 0 until container.containerSize) {
				val stack = container.getItem(i)

				if (!stack.isEmpty) {
					stack.energy?.let {
						result += it.maxEnergyStoredMattery
					}
				}
			}

			return result
		}
	}

	private val energyReceiver = BatteryBankEnergy(BankMode.RECEIVE)
	private var resolverEnergyReceive = LazyOptional.of { energyReceiver }
	private val energyExtractor = BatteryBankEnergy(BankMode.EXTRACT)
	private var resolverEnergyExtractor = LazyOptional.of { energyExtractor }
	private val energy = BatteryBankEnergy(BankMode.BIDIRECTIONAL)
	private var resolverEnergy = LazyOptional.of { energy }
	private var resolverItemHandler = LazyOptional.of { itemHandler }

	private var valid = true

	override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)
		nbt["container"] = container.serializeNBT()
	}

	override fun load(nbt: CompoundTag) {
		container.deserializeNBT(nbt["container"])
		super.load(nbt)
	}

	override val defaultDisplayName: Component
		get() = MACHINE_NAME

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return BatteryBankMenu(containerID, inventory, this)
	}

	override fun invalidateCaps() {
		super.invalidateCaps()
		valid = false
		resolverEnergyReceive.invalidate()
		resolverEnergyExtractor.invalidate()
		resolverEnergy.invalidate()
		resolverItemHandler.invalidate()
	}

	override fun reviveCaps() {
		super.reviveCaps()
		valid = true
		resolverEnergyReceive = LazyOptional.of { energyReceiver }
		resolverEnergyExtractor = LazyOptional.of { energyExtractor }
		resolverEnergy = LazyOptional.of { energy }
		resolverItemHandler = LazyOptional.of { itemHandler }
	}

	override fun setLevel(p_155231_: Level) {
		super.setLevel(p_155231_)
		tickOnceServer { level: Level -> checkSurroundings(level) }
	}

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		if (valid) {
			if (cap === MatteryCapability.ENERGY || cap === CapabilityEnergy.ENERGY) {
				if (side == null) return resolverEnergy.cast()

				if (side == blockState.getValue(RotatableMatteryBlock.FACING))
					return resolverEnergyExtractor.cast()
				else
					return resolverEnergyReceive.cast()
			}

			if (cap === CapabilityItemHandler.ITEM_HANDLER_CAPABILITY) {
				return resolverItemHandler.cast()
			}
		}

		return super.getCapability(cap,side)
	}

	private var outputCapability = LazyOptional.empty<IEnergyStorage>()

	fun checkSurroundings(level: Level) {
		if (isRemoved) return

		val tile = level.getBlockEntity(blockPos.offset(blockState.getValue(RotatableMatteryBlock.FACING).normal))

		if (tile == null) {
			outputCapability = LazyOptional.empty()
			return
		}

		outputCapability = getAndBind(
			outputCapability,
			tile,
			CapabilityEnergy.ENERGY,
			-blockState.getValue(RotatableMatteryBlock.FACING)
		) {
			@Suppress("name_shadowing")
			val level = this.level

			if (level is ServerLevel && !SERVER_IS_DYING)
				checkSurroundings(level)
		}
	}

	fun tick() {
		if (isBlockedByRedstone)
			return

		outputCapability.ifPresentK {
			val (_, maxThroughput) = energy.getDistribution(false)

			if (maxThroughput.isZero)
				return@ifPresentK

			val diff = it.receiveEnergy(maxThroughput, false)

			if (!diff.isZero) {
				energy.extractEnergyInner(diff, false)
			}
		}
	}

	companion object {
		private val MACHINE_NAME = TranslatableComponent("block.overdrive_that_matters.battery_bank")
		const val CAPACITY = 6 * 2
	}
}