package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.network.chat.TranslatableComponent
import net.minecraft.server.level.ServerLevel
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.state.BlockState
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.util.LazyOptional
import ru.dbotthepony.mc.otm.graph.storage.BasicStorageGraphNode
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.WorkerEnergyStorage
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.ifHas
import ru.dbotthepony.mc.otm.menu.DriveRackMenu
import ru.dbotthepony.mc.otm.set
import ru.dbotthepony.mc.otm.graph.storage.StorageNetworkGraph
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import ru.dbotthepony.mc.otm.storage.*

class DriveRackBlockEntity(p_155229_: BlockPos, p_155230_: BlockState) :
	MatteryPoweredBlockEntity(MBlockEntities.DRIVE_RACK, p_155229_, p_155230_) {
	override val energy = WorkerEnergyStorage(this, STORAGE)

	@JvmField
	val drives: MatteryContainer = object : MatteryContainer(this::setChanged, 4) {
		override fun setChanged(slot: Int, new: ItemStack, old: ItemStack) {
			super.setChanged(slot, new, old)

			old.getCapability(MatteryCapability.DRIVE).ifPresent {
				cell.computeIfAbsent(it.storageType) {
					PoweredVirtualComponent(it, energy)
				}.remove(it)
			}

			new.getCapability(MatteryCapability.DRIVE).ifPresent {
				cell.computeIfAbsent(it.storageType) {
					PoweredVirtualComponent(it, energy)
				}.add(it)
			}
		}
	}

	val cell = BasicStorageGraphNode()

	override fun load(nbt: CompoundTag) {
		super.load(nbt)

		nbt.ifHas("drives", CompoundTag::class.java) {
			drives.deserializeNBT(it)
		}
	}

	override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)
		nbt["drives"] = drives.serializeNBT()
	}

	override fun setLevel(p_155231_: Level) {
		super.setLevel(p_155231_)

		if (p_155231_ is ServerLevel)
			StorageNetworkGraph.discoverFull(this, cell.storageNode)
	}

	fun tick() {
		batteryChargeLoop()
	}

	override val defaultDisplayName: Component
		get() = MACHINE_NAME

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return DriveRackMenu(containerID, inventory, this)
	}

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		return if (cap === MatteryCapability.STORAGE_NODE) {
			cell.get().cast()
		} else super.getCapability(cap, side)
	}

	override fun invalidateCaps() {
		super.invalidateCaps()
		cell.invalidate()
	}

	override fun reviveCaps() {
		super.reviveCaps()
		cell.revive()
	}

	override fun setRemoved() {
		super.setRemoved()
		cell.destroy(level)
	}

	companion object {
		private val MACHINE_NAME = TranslatableComponent("block.overdrive_that_matters.drive_rack")
		private val STORAGE = ImpreciseFraction(80_000)
	}
}