package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.server.level.ServerLevel
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraft.world.level.block.state.BlockState
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.util.LazyOptional
import ru.dbotthepony.mc.otm.block.BlockMatterCable
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.graph.Graph6Node
import ru.dbotthepony.mc.otm.graph.GraphNodeListener
import ru.dbotthepony.mc.otm.graph.matter.IMatterGraphNode
import ru.dbotthepony.mc.otm.graph.matter.MatterNetworkGraph
import ru.dbotthepony.mc.otm.registry.MBlockEntities

class MatterCableBlockEntity(p_155229_: BlockPos, p_155230_: BlockState) :
	BlockEntity(MBlockEntities.MATTER_CABLE, p_155229_, p_155230_), IMatterGraphNode, GraphNodeListener {

	private var valid = true
	private val node = Graph6Node<IMatterGraphNode>(this)
	private val resolverNode = LazyOptional.of { this }

	override fun getAsMatterNode(): Graph6Node<IMatterGraphNode> {
		return node
	}

	override fun invalidateCaps() {
		super.invalidateCaps()
		valid = false
	}

	override fun reviveCaps() {
		super.reviveCaps()
		valid = true
	}

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		if (valid) {
			if (cap === MatteryCapability.MATTER_NODE)
				return resolverNode.cast()
		}

		return super.getCapability(cap, side)
	}

	override fun setLevel(p_155231_: Level) {
		super.setLevel(p_155231_)

		if (p_155231_ is ServerLevel)
			MatterNetworkGraph.discoverFull(this, node)
	}

	override fun onNeighbour(node: Graph6Node<*>, direction: Direction) {
		val new_state = blockState.setValue(BlockMatterCable.MAPPING_CONNECTION_PROP[direction.ordinal], true)
		if (new_state !== blockState) level!!.setBlock(blockPos, new_state, Block.UPDATE_CLIENTS)
	}

	override fun onUnNeighbour(node: Graph6Node<*>, direction: Direction) {
		val new_state = blockState.setValue(BlockMatterCable.MAPPING_CONNECTION_PROP[direction.ordinal], false)
		if (new_state !== blockState) level!!.setBlock(blockPos, new_state, Block.UPDATE_CLIENTS)
	}

	override fun setRemoved() {
		super.setRemoved()
		node.destroy(::MatterNetworkGraph)
	}
}