package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.world.level.block.entity.BlockEntityType
import net.minecraft.core.BlockPos
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraft.world.MenuProvider
import java.lang.Runnable
import net.minecraft.server.level.ServerLevel
import net.minecraft.client.multiplayer.ClientLevel
import net.minecraft.core.Direction
import net.minecraft.core.SectionPos
import net.minecraftforge.common.util.LazyOptional
import net.minecraftforge.common.capabilities.ICapabilityProvider
import java.lang.ref.WeakReference
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.nbt.CompoundTag
import net.minecraft.nbt.StringTag
import net.minecraft.nbt.ByteTag
import net.minecraft.network.chat.Component
import net.minecraft.world.level.Level
import net.minecraftforge.common.capabilities.Capability
import ru.dbotthepony.mc.otm.addPreWorldTickerOnce
import ru.dbotthepony.mc.otm.ifHas
import ru.dbotthepony.mc.otm.set

abstract class MatteryBlockEntity(p_155228_: BlockEntityType<*>, p_155229_: BlockPos, p_155230_: BlockState) : BlockEntity(p_155228_, p_155229_, p_155230_), MenuProvider {
	var customDisplayName: Component? = null
	var redstoneSignal = 0
		set(level) {
			val old = isBlockedByRedstone
			field = level
			val state = isBlockedByRedstone

			if (old != state) {
				redstoneStatusUpdated(state, old)
			}
		}

	var redstoneSetting: RedstoneSetting = RedstoneSetting.LOW
		set(setting) {
			val old = isBlockedByRedstone
			field = setting
			val state = isBlockedByRedstone
			if (old != state) {
				redstoneStatusUpdated(state, old)
			}
		}

	val isBlockedByRedstone: Boolean
		get() {
			when (redstoneSetting) {
				RedstoneSetting.LOW -> {
					return redstoneSignal > 0
				}

				RedstoneSetting.HIGH -> {
					return redstoneSignal <= 0
				}

				RedstoneSetting.IGNORED -> {
					return false
				}
			}
		}

	protected abstract val defaultDisplayName: Component
	abstract override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu?

	protected fun tickOnce(func: Runnable) {
		val level = level
		if (level != null) addPreWorldTickerOnce(level) { if (!isRemoved) func.run() }
		}

	protected fun tickOnceServer(func: Runnable) {
		val level = level
		if (level is ServerLevel) addPreWorldTickerOnce(level) { if (!isRemoved) func.run() }
	}

	protected fun tickOnceClient(func: Runnable) {
		val level = level
		if (level is ClientLevel) addPreWorldTickerOnce(level) { if (!isRemoved) func.run() }
	}

	protected fun tickOnce(func: (Level) -> Unit) {
		val level = level
		if (level != null) addPreWorldTickerOnce(level) { if (!isRemoved) func(level) }
	}

	protected fun tickOnceServer(func: (ServerLevel) -> Unit) {
		val level = level
		if (level is ServerLevel) addPreWorldTickerOnce(level) { if (!isRemoved) func(level) }
	}

	protected fun tickOnceClient(func: (ClientLevel) -> Unit) {
		val level = level
		if (level is ClientLevel) addPreWorldTickerOnce(level) { if (!isRemoved) func(level) }
	}

	protected fun <T> getAndBind(
		old: LazyOptional<T>,
		provider: ICapabilityProvider?,
		cap: Capability<T>,
		side: Direction,
		invalidate: Runnable
	): LazyOptional<T> {
		val get = provider?.getCapability(cap, side) ?: LazyOptional.empty()

		if (old !== get) {
			if (get.isPresent) {
				val ref = WeakReference(invalidate)
				get.addListener {
					ref.get()?.run()
				}
			}

			return get.cast()
		}

		return get.cast()
	}

	override fun getDisplayName(): Component {
		return customDisplayName ?: defaultDisplayName
	}

	protected open fun redstoneStatusUpdated(new_blocked: Boolean, old_blocked: Boolean) {}

	override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)

		val customDisplayName = customDisplayName
		if (customDisplayName != null)
			nbt.putString("Name", Component.Serializer.toJson(customDisplayName))

		nbt["redstone"] = redstoneSetting.ordinal.toByte()
		nbt["redstone_signal"] = redstoneSignal.toByte()
	}

	override fun load(nbt: CompoundTag) {
		super.load(nbt)

		nbt.ifHas("Name", StringTag::class.java) {
			customDisplayName = Component.Serializer.fromJson(it.asString)
		}

		nbt.ifHas("redstone", ByteTag::class.java) {
			redstoneSetting = RedstoneSetting.get(it.asInt)
		}

		nbt.ifHas("redstone_signal", ByteTag::class.java) {
			redstoneSignal = it.asInt
		}
	}

	// Just to mark chunk unsaved
	open fun setChangedLight() {
		val level = level
		if (level is ServerLevel) {
			level.chunkSource.getChunkNow(
				SectionPos.blockToSectionCoord(blockPos.x),
				SectionPos.blockToSectionCoord(blockPos.z))?.isUnsaved = true
		}
	}
}
