package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.nbt.CompoundTag
import net.minecraft.world.level.block.entity.BlockEntityType
import net.minecraft.world.level.block.state.BlockState
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.util.LazyOptional
import net.minecraftforge.energy.CapabilityEnergy
import ru.dbotthepony.mc.otm.capability.*
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.ifHas
import ru.dbotthepony.mc.otm.set

abstract class MatteryPoweredBlockEntity(p_155228_: BlockEntityType<*>, p_155229_: BlockPos, p_155230_: BlockState) : MatteryBlockEntity(p_155228_, p_155229_, p_155230_) {
	abstract val energy: BlockEnergyStorageImpl
	private var valid = true
	val batteryContainer = MatteryContainer(this::setChangedLight, 1)

	protected fun batteryChargeLoop() {
		var demand = energy.receiveEnergyOuter(energy.missingPower, true)
		if (demand.isZero) return

		for (stack in batteryContainer) {
			if (!stack.isEmpty) {
				stack.getCapability(CapabilityEnergy.ENERGY).ifPresent {
					if (it is IMatteryEnergyStorage) {
						val diff = it.extractEnergyOuter(demand, false)
						energy.receiveEnergyInner(diff, false)
						demand -= diff
					} else {
						val diff = it.extractEnergy(demand, false)
						energy.receiveEnergyInner(diff, false)
						demand -= diff
					}
				}

				if (demand <= ImpreciseFraction.ZERO)
					return
			}
		}
	}

	override fun invalidateCaps() {
		super.invalidateCaps()
		energy.invalidate()
		valid = false
	}

	override fun reviveCaps() {
		super.reviveCaps()
		energy.revive()
		valid = true
	}

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		if (valid && (cap === MatteryCapability.ENERGY || cap === CapabilityEnergy.ENERGY))
			return energy.resolver.cast()

		return super.getCapability(cap, side)
	}

	public override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)
		nbt["energy"] = energy.serializeNBT()
		nbt["battery_container"] = batteryContainer.serializeNBT()
	}

	override fun load(nbt: CompoundTag) {
		nbt.ifHas("energy", CompoundTag::class.java, energy::deserializeNBT)
		nbt.ifHas("battery_container", CompoundTag::class.java, batteryContainer::deserializeNBT)
		super.load(nbt)
	}
}