package ru.dbotthepony.mc.otm.block.entity.worker

import net.minecraft.nbt.CompoundTag
import net.minecraft.nbt.DoubleTag
import net.minecraft.nbt.Tag
import net.minecraft.util.StringRepresentable
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.block.state.properties.EnumProperty
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.set

@JvmRecord
data class WorkerTickContext(
	val job: WorkerJob,
	val requiredPower: ImpreciseFraction,
	val extractedPower: ImpreciseFraction,
	val ticksAdvanced: Double
)

enum class WorkerState : StringRepresentable {
	IDLE,
	WORKING,
	ERROR;

	companion object {
		@JvmField
		val WORKER_STATE: EnumProperty<WorkerState> = EnumProperty.create("worker", WorkerState::class.java)
		@JvmField
		val SEMI_WORKER_STATE: EnumProperty<WorkerState> = EnumProperty.create("worker", WorkerState::class.java, IDLE, WORKING)
	}

	override fun getSerializedName(): String {
		return if (this == IDLE) "idle" else if (this == WORKING) "working" else "error"
	}
}

@JvmRecord
data class WorkerJobStatus @JvmOverloads constructor(val valid: Boolean = true, val throttle: Int = 0) {
	constructor(throttle: Int) : this(false, throttle)
}

@JvmRecord
data class WorkerJob @JvmOverloads constructor(
	val stack: ItemStack,
	val ticks: Double,
	val power: ImpreciseFraction = ImpreciseFraction.ZERO,
	val data: CompoundTag = CompoundTag()) {

	fun serializeNBT(): CompoundTag {
		return CompoundTag().also {
			it["stack"] = stack.serializeNBT()
			it["ticks"] = ticks
			it["power"] = power.serializeNBT()
			it["data"] = data
		}
	}

	operator fun get(index: String) = data[index]
	operator fun set(index: String, value: Tag) { data[index] = value }
	operator fun set(index: String, value: Int) { data[index] = value }
	operator fun set(index: String, value: Byte) { data[index] = value }
	operator fun set(index: String, value: Short) { data[index] = value }
	operator fun set(index: String, value: Long) { data[index] = value }
	operator fun set(index: String, value: Float) { data[index] = value }
	operator fun set(index: String, value: Double) { data[index] = value }
	operator fun set(index: String, value: String) { data[index] = value }
	operator fun set(index: String, value: Boolean) { data[index] = value }
	operator fun set(index: String, value: ByteArray) { data[index] = value }
	operator fun set(index: String, value: IntArray) { data[index] = value }
	operator fun set(index: String, value: LongArray) { data[index] = value }

	companion object {
		@JvmStatic
		fun deserializeNBT(tag: Tag?): WorkerJob? {
			val nbt = tag as? CompoundTag ?: return null

			return WorkerJob(
				ItemStack.of(nbt["stack"] as? CompoundTag ?: return null),
				(nbt["ticks"] as? DoubleTag ?: return null).asDouble,
				ImpreciseFraction.deserializeNBT(nbt["power"]),
				nbt["data"] as? CompoundTag ?: return null
			)
		}
	}
}
