package ru.dbotthepony.mc.otm.capability

import net.minecraft.core.Direction
import net.minecraft.world.entity.LivingEntity
import net.minecraftforge.common.capabilities.ICapabilityProvider
import net.minecraftforge.common.util.LazyOptional
import net.minecraftforge.energy.CapabilityEnergy
import net.minecraftforge.energy.IEnergyStorage
import net.minecraftforge.fml.ModList
import ru.dbotthepony.mc.otm.compat.mekanism.getMekanismEnergySided
import ru.dbotthepony.mc.otm.compat.mekanism.mekanismEnergy
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.orNull

fun IEnergyStorage.receiveEnergy(amount: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
	if (this is IMatteryEnergyStorage)
		return receiveEnergyOuter(amount, simulate)

	if (!amount.isPositive)
		return ImpreciseFraction.ZERO

	if (amount > ImpreciseFraction.INT_MAX_VALUE)
		return ImpreciseFraction.valueOf(receiveEnergy(Int.MAX_VALUE, simulate))

	return ImpreciseFraction.valueOf(receiveEnergy(amount.toInt(), simulate))
}

fun IEnergyStorage.extractEnergy(amount: ImpreciseFraction, simulate: Boolean): ImpreciseFraction {
	if (this is IMatteryEnergyStorage)
		return extractEnergyOuter(amount, simulate)

	if (!amount.isPositive)
		return ImpreciseFraction.ZERO

	if (amount > ImpreciseFraction.INT_MAX_VALUE)
		return ImpreciseFraction.valueOf(extractEnergy(Int.MAX_VALUE, simulate))

	return ImpreciseFraction.valueOf(extractEnergy(amount.toInt(), simulate))
}

val IEnergyStorage.maxEnergyStoredMattery: ImpreciseFraction get() {
	if (this is IMatteryEnergyStorage) {
		return maxBatteryLevel
	}

	return ImpreciseFraction.valueOf(maxEnergyStored)
}

val IEnergyStorage.energyStoredMattery: ImpreciseFraction get() {
	if (this is IMatteryEnergyStorage) {
		return batteryLevel
	}

	return ImpreciseFraction.valueOf(energyStored)
}

val isMekanismLoaded by lazy { ModList.get().isLoaded("mekanism") }

/**
 * Shortcut for getting [IEnergyStorage], including wrappers for it
 */
val ICapabilityProvider.energy: IEnergyStorage? get() {
	if (isMekanismLoaded) {
		val mekanismEnergy = mekanismEnergy

		if (mekanismEnergy != null) {
			return mekanismEnergy
		}
	}

	return getCapability(CapabilityEnergy.ENERGY).orNull()
}

/**
 * Shortcut for getting [IMatteryEnergyStorage], including wrappers for it
 */
val ICapabilityProvider.matteryEnergy: IMatteryEnergyStorage? get() {
	if (isMekanismLoaded) {
		val mekanismEnergy = mekanismEnergy

		if (mekanismEnergy != null) {
			return mekanismEnergy
		}
	}

	return getCapability(MatteryCapability.ENERGY).orNull()
}

/**
 * Shortcut for getting [LazyOptional] with [IEnergyStorage], including wrappers for it
 */
fun ICapabilityProvider.getEnergySided(side: Direction? = null): LazyOptional<IEnergyStorage> {
	if (isMekanismLoaded) {
		val mekanismEnergy = getMekanismEnergySided(side)

		if (mekanismEnergy.isPresent) {
			return mekanismEnergy.cast()
		}
	}

	return getCapability(CapabilityEnergy.ENERGY, side)
}

/**
 * Shortcut for getting [LazyOptional] with [IMatteryEnergyStorage], including wrappers for it
 */
fun ICapabilityProvider.getMatteryEnergySided(side: Direction? = null): LazyOptional<IMatteryEnergyStorage> {
	if (isMekanismLoaded) {
		val mekanismEnergy = getMekanismEnergySided(side)

		if (mekanismEnergy.isPresent) {
			return mekanismEnergy
		}
	}

	return getCapability(MatteryCapability.ENERGY, side)
}

val LivingEntity.android get() = getCapability(MatteryCapability.ANDROID).orNull()
