package ru.dbotthepony.mc.otm.capability.drive

import net.minecraft.nbt.CompoundTag
import net.minecraft.resources.ResourceLocation
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.Items
import net.minecraftforge.registries.RegistryManager
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.set
import ru.dbotthepony.mc.otm.storage.IStorageTuple
import ru.dbotthepony.mc.otm.storage.ItemStackWrapper
import ru.dbotthepony.mc.otm.storage.StorageStackType
import java.util.*

class ItemMatteryDrive : AbstractMatteryDrive<ItemStackWrapper>, IItemMatteryDrive {
	constructor(capacity: ImpreciseFraction, max_different_stacks: Int) : super(capacity, maxDifferentStacks = max_different_stacks)
	constructor(capacity: ImpreciseFraction, uuid: UUID, max_different_stacks: Int) : super(capacity, uuid, max_different_stacks)
	constructor(capacity: ImpreciseFraction, uuid: UUID) : super(capacity, uuid)
	constructor(capacity: ImpreciseFraction) : super(capacity)

	override val storageType: StorageStackType<ItemStackWrapper> = OverdriveThatMatters.INSTANCE.ITEM_STORAGE()

	fun insertStack(item: ItemStack, simulate: Boolean): ItemStack {
		return insertStack(ItemStackWrapper(item), simulate).stack
	}

	override fun serializeStack(item: IStorageTuple<ItemStackWrapper>): CompoundTag? {
		val tag = CompoundTag()
		val location = item.stack.registryName.toString()

		tag["item"] = location
		tag["count"] = item.stack.count.serializeNBT()

		val itag = item.stack.item.tag

		if (itag != null) {
			tag["data"] = itag
		}

		return tag
	}

	override fun deserializeStack(tag: CompoundTag): ItemStackWrapper? {
		val item = RegistryManager.ACTIVE.getRegistry(Item::class.java).getValue(ResourceLocation(tag.getString("item")))

		if (item != null && item !== Items.AIR) {
			val count = tag.getInt("count")
			val itemstack = ItemStack(item, count)
			itemstack.tag = tag["data"] as? CompoundTag?
			return ItemStackWrapper(itemstack)
		}

		return null
	}

	override fun findItems(item: Item): List<IStorageTuple<ItemStackWrapper>> {
		val list = storedStacks[ItemStackWrapper.longHashCode(item)]
		return if (list != null) java.util.List.copyOf(list) else emptyList()
	}

	override fun findItems(stack: ItemStack): List<IStorageTuple<ItemStackWrapper>> {
		val list = storedStacks[ItemStackWrapper.longHashCode(stack.item)] ?: return emptyList()

		val buildList = ArrayList<IStorageTuple<ItemStackWrapper>>()

		for (_stack in list) {
			if (_stack.stack.sameItem(stack)) {
				buildList.add(_stack)
			}
		}

		return buildList
	}

	companion object {
		@JvmField
		val DUMMY = ItemMatteryDrive(ImpreciseFraction(0), UUID(0L, 0L), 0)
	}
}
