package ru.dbotthepony.mc.otm.client.render

import com.mojang.blaze3d.systems.RenderSystem
import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.resources.ResourceLocation

@JvmRecord
data class SkinElement @JvmOverloads constructor(
	val texture: ResourceLocation = RenderHelper.WIDGETS,
	val image_x: Float,
	val image_y: Float,
	val rect_w: Float,
	val rect_h: Float,
	val defined_width: Float = 256f,
	val defined_height: Float = 256f
) {
	@JvmOverloads
	fun render(
		stack: PoseStack,
		x: Float = 0f,
		y: Float = 0f,
		width: Float = rect_w,
		height: Float = rect_h,
		winding: UVWindingOrder = UVWindingOrder.U0_V0_U1_V1
	) {
		RenderSystem.setShaderTexture(0, texture)
		RenderSystem.enableBlend()
		RenderSystem.defaultBlendFunc()

		renderRaw(stack, x, y, width, height, winding)
	}

	@JvmOverloads
	fun render(
		stack: PoseStack,
		x: Double,
		y: Double = 0.0,
		width: Double = rect_w.toDouble(),
		height: Double = rect_h.toDouble(),
		winding: UVWindingOrder = UVWindingOrder.U0_V0_U1_V1
	) = render(stack, x.toFloat(), y.toFloat(), width.toFloat(), height.toFloat(), winding)

	@JvmOverloads
	fun renderPartial(
		stack: PoseStack,
		x: Float = 0f,
		y: Float = 0f,
		width: Float = rect_w,
		height: Float = rect_h,
		winding: UVWindingOrder = UVWindingOrder.U0_V0_U1_V1
	) {
		RenderSystem.setShaderTexture(0, texture)
		RenderSystem.enableBlend()
		RenderSystem.defaultBlendFunc()

		renderRawPartial(stack, x, y, width, height, winding)
	}

	@JvmOverloads
	fun renderPartial(
		stack: PoseStack,
		x: Double,
		y: Double = 0.0,
		width: Double = rect_w.toDouble(),
		height: Double = rect_h.toDouble(),
		winding: UVWindingOrder = UVWindingOrder.U0_V0_U1_V1
	) = renderPartial(stack, x.toFloat(), y.toFloat(), width.toFloat(), height.toFloat(), winding)

	@JvmOverloads
	fun renderW(
		stack: PoseStack,
		x: Float = 0f,
		y: Float = 0f,
		width: Float = rect_w,
		winding: UVWindingOrder = UVWindingOrder.U0_V0_U1_V1,
	) {
		render(stack, x, y, width = width, winding = winding)
	}

	@JvmOverloads
	fun renderW(
		stack: PoseStack,
		x: Double,
		y: Double = 0.0,
		width: Double = rect_w.toDouble(),
		winding: UVWindingOrder = UVWindingOrder.U0_V0_U1_V1,
	) = renderW(stack, x.toFloat(), y.toFloat(), width.toFloat(), winding)

	@JvmOverloads
	fun renderH(
		stack: PoseStack,
		x: Float = 0f,
		y: Float = 0f,
		height: Float = rect_h,
		winding: UVWindingOrder = UVWindingOrder.U0_V0_U1_V1,
	) {
		render(stack, x, y, height = height, winding = winding)
	}

	@JvmOverloads
	fun renderH(
		stack: PoseStack,
		x: Double,
		y: Double = 0.0,
		height: Double = rect_h.toDouble(),
		winding: UVWindingOrder = UVWindingOrder.U0_V0_U1_V1,
	) = renderH(stack, x.toFloat(), y.toFloat(), height.toFloat(), winding)

	@JvmOverloads
	fun renderRaw(
		stack: PoseStack,
		x: Float = 0f,
		y: Float = 0f,
		width: Float = rect_w,
		height: Float = rect_h,
		winding: UVWindingOrder = UVWindingOrder.U0_V0_U1_V1
	) {
		val u0 = image_x / defined_width
		val v0 = image_y / defined_height
		val u1 = (image_x + rect_w) / defined_width
		val v1 = (image_y + rect_h) / defined_height

		val winded = winding.translate(u0, v0, u1, v1)

		RenderHelper.drawTexturedRectUV(
			stack,
			x,
			y,
			width,
			height,
			winded.u0,
			winded.v0,
			winded.u1,
			winded.v1,
		)
	}

	@JvmOverloads
	fun renderRaw(
		stack: PoseStack,
		x: Double,
		y: Double = 0.0,
		width: Double = rect_w.toDouble(),
		height: Double = rect_h.toDouble(),
		winding: UVWindingOrder = UVWindingOrder.U0_V0_U1_V1
	) = renderRaw(stack, x.toFloat(), y.toFloat(), width.toFloat(), height.toFloat(), winding)

	@JvmOverloads
	fun renderRawPartial(
		stack: PoseStack,
		x: Float = 0f,
		y: Float = 0f,
		width: Float = rect_w,
		height: Float = rect_h,
		winding: UVWindingOrder = UVWindingOrder.U0_V0_U1_V1
	) {
		val u0 = image_x / defined_width
		val v0 = image_y / defined_height
		val u1 = (image_x + width) / defined_width
		val v1 = (image_y + height) / defined_height

		val winded = winding.translate(u0, v0, u1, v1)

		RenderHelper.drawTexturedRectUV(
			stack,
			x,
			y,
			width,
			height,
			winded.u0,
			winded.v0,
			winded.u1,
			winded.v1,
		)
	}

	@JvmOverloads
	fun renderRawPartial(
		stack: PoseStack,
		x: Double,
		y: Double = 0.0,
		width: Double = rect_w.toDouble(),
		height: Double = rect_h.toDouble(),
		winding: UVWindingOrder = UVWindingOrder.U0_V0_U1_V1
	) = renderRawPartial(stack, x.toFloat(), y.toFloat(), width.toFloat(), height.toFloat(), winding)
}

@JvmField val top_left_window_corner = SkinElement(image_x = 18f, image_y = 0f, rect_w = 6f, rect_h = 6f)
@JvmField val top_right_window_corner = SkinElement(image_x = 24f, image_y = 0f, rect_w = 6f, rect_h = 6f)
@JvmField val left_window_border = SkinElement(image_x = 18f, image_y = 4f, rect_w = 3f, rect_h = 5f)
@JvmField val right_window_border = SkinElement(image_x = 27f, image_y = 3f, rect_w = 3f, rect_h = 5f)
@JvmField val top_window_border = SkinElement(image_x = 22f, image_y = 0f, rect_w = 5f, rect_h = 3f)
@JvmField val window_background = SkinElement(image_x = 30f, image_y = 12f, rect_w = 6f, rect_h = 6f)
@JvmField val scroll_bar_top = SkinElement(image_x = 18f, image_y = 57f, rect_w = 14f, rect_h = 2f)
@JvmField val scroll_bar_bottom = SkinElement(image_x = 18f, image_y = 63f, rect_w = 14f, rect_h = 2f)
@JvmField val scroll_bar_body = SkinElement(image_x = 18f, image_y = 58f, rect_w = 14f, rect_h = 6f)
@JvmField val scroll_bar_button = SkinElement(image_x = 18f, image_y = 12f, rect_w = 12f, rect_h = 15f)
@JvmField val scroll_bar_button_hover = SkinElement(image_x = 18f, image_y = 27f, rect_w = 12f, rect_h = 15f)
@JvmField val scroll_bar_button_press = SkinElement(image_x = 18f, image_y = 42f, rect_w = 12f, rect_h = 15f)
@JvmField val tab_right_connection = SkinElement(image_x = 30f, image_y = 0f, rect_w = 3f, rect_h = 5f)
@JvmField val tab_left_connection = SkinElement(image_x = 33f, image_y = 0f, rect_w = 3f, rect_h = 5f)
@JvmField val tab_background = SkinElement(image_x = 30f, image_y = 6f, rect_w = 6f, rect_h = 6f)
@JvmField val bottom_left_window_corner = SkinElement(image_x = 18f, image_y = 6f, rect_w = 6f, rect_h = 6f)
@JvmField val bottom_right_window_corner = SkinElement(image_x = 24f, image_y = 6f, rect_w = 6f, rect_h = 6f)
@JvmField val bottom_window_border = SkinElement(image_x = 21f, image_y = 9f, rect_w = 5f, rect_h = 3f)
