package ru.dbotthepony.mc.otm.client.screen

import net.minecraft.network.chat.Component
import net.minecraft.network.chat.TranslatableComponent
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.client.screen.panels.*
import ru.dbotthepony.mc.otm.client.screen.widget.PowerGaugePanel
import ru.dbotthepony.mc.otm.item.PortableCondensationDriveItem.FilterSettings
import ru.dbotthepony.mc.otm.menu.DriveViewerMenu
import ru.dbotthepony.mc.otm.menu.DriveViewerMenu.FilterSetPacket
import ru.dbotthepony.mc.otm.menu.DriveViewerMenu.FilterSwitchPacket
import ru.dbotthepony.mc.otm.network.MatteryNetworking

class DriveViewerScreen(menu: DriveViewerMenu, inventory: Inventory, title: Component) :
	MatteryScreen<DriveViewerMenu>(menu, inventory, title) {

	override fun makeMainFrame(): FramePanel {
		val frame = FramePanel(this, null, 0f, 0f, FRAME_WIDTH, FRAME_HEIGHT, getTitle())

		val views = ArrayList<EditablePanel>()
		val settings = ArrayList<EditablePanel>()
		val viewButton = frame.addTab(FramePanel.Position.TOP)
		val settingsButton = frame.addTab(FramePanel.Position.TOP)

		viewButton.bindOnOpen {
			for (panel in views) {
				panel.visible = true
			}
		}

		viewButton.bindOnClose {
			for (panel in views) {
				panel.visible = false
			}
		}

		settingsButton.bindOnOpen {
			for (panel in settings) {
				panel.visible = true
			}
		}

		settingsButton.bindOnClose {
			for (panel in settings) {
				panel.visible = false
			}
		}

		views.add(PowerGaugePanel(this, frame, menu.powerWidget, 8f, 16f))
		views.add(SlotPanel(this, frame, menu.batterySlot, 8f, 67f))
		views.add(SlotPanel(this, frame, menu.driveSlot, 8f, 85f))

		val grid = GridPanel(this, frame, 28f, 16f, GRID_WIDTH * 18f, GRID_HEIGHT * 18f, GRID_WIDTH, GRID_HEIGHT)

		val scroll_bar = ScrollBarPanel(this, frame, 192f, 14f, 92f)
		scroll_bar.setupRowMultiplier { menu.view.getItemCount() / GRID_WIDTH }
		views.add(grid)
		views.add(scroll_bar)

		for (i in 0 until GRID_WIDTH * GRID_HEIGHT) {
			object : AbstractSlotPanel(this@DriveViewerScreen, grid, 0f, 0f) {
				override fun getItemStack(): ItemStack {
					val index = i + scroll_bar.getScroll(menu.view.getItems().size / GRID_WIDTH)
					val list = menu.view.getItems()

					return if (index >= list.size) {
						ItemStack.EMPTY
					} else list[index].stack
				}

				override fun mouseScrolledInner(mouse_x: Double, mouse_y: Double, scroll: Double): Boolean {
					return scroll_bar.mouseScrolledInner(mouse_x, mouse_y, scroll)
				}

				override fun mouseClickedInner(mouse_x: Double, mouse_y: Double, mouse_click_type: Int): Boolean {
					val index = i + scroll_bar.getScroll(GRID_WIDTH)
					menu.view.mouseClick(index, mouse_click_type)
					return true
				}
			}
		}

		val dock_left = FlexGridPanel(this, frame, 0f, 0f, 90f, 0f)
		dock_left.dock = Dock.LEFT
		dock_left.setDockMargin(4f, 0f, 4f, 0f)

		val grid_filter = FlexGridPanel(this, frame)
		grid_filter.dock = Dock.FILL
		settings.add(dock_left)
		settings.add(grid_filter)

		for (i in 0 until FilterSettings.MAX_FILTERS) {
			object : AbstractSlotPanel(this@DriveViewerScreen, grid_filter, 0f, 0f) {
				override fun getItemStack(): ItemStack {
					val filter = menu.getFilter() ?: return ItemStack.EMPTY
					return filter.items[i]
				}

				private var clicking = false

				override fun mouseClickedInner(mouse_x: Double, mouse_y: Double, mouse_click_type: Int): Boolean {
					clicking = true
					return true
				}

				override fun mouseReleasedInner(mouse_x: Double, mouse_y: Double, flag: Int): Boolean {
					if (clicking)
						MatteryNetworking.send(null, FilterSetPacket(menu.containerId, i, menu.carried))

					clicking = false
					return true
				}
			}
		}

		val no = TranslatableComponent("otm.filter.no")
		val yes = TranslatableComponent("otm.filter.yes")

		val matchNbt =
			object : ButtonPanel(this@DriveViewerScreen, dock_left, 0f, 0f, 90f, 20f, TranslatableComponent("otm.filter.match_nbt", no)) {
				override fun tick() {
					super.tick()

					val filter = menu.getFilter()
					if (filter != null) {
						widget?.message = TranslatableComponent("otm.filter.match_nbt", if (filter.matchNbt) yes else no)
						widget?.active = !isWidgetDisabled
					} else {
						widget?.active = false
					}
				}

				override fun onPress() {
					super.onPress()

					val filter = menu.getFilter()
					if (filter != null) {
						MatteryNetworking.send(
							null,
							FilterSwitchPacket(
								menu.containerId,
								DriveViewerMenu.FilterSwitch.MATCH_NBT,
								!filter.matchNbt
							)
						)

						disableTicks = 20
					}
				}
			}

		val matchTag =
			object : ButtonPanel(this@DriveViewerScreen, dock_left, 0f, 0f, 90f, 20f, TranslatableComponent("otm.filter.match_tag", no)) {
				override fun tick() {
					super.tick()

					val filter = menu.getFilter()
					if (filter != null) {
						widget?.message = TranslatableComponent("otm.filter.match_tag", if (filter.matchTag) yes else no)
						widget?.active = !isWidgetDisabled
					} else {
						widget?.active = false
					}
				}

				override fun onPress() {
					super.onPress()

					val filter = menu.getFilter()
					if (filter != null) {
						MatteryNetworking.send(
							null,
							FilterSwitchPacket(
								menu.containerId,
								DriveViewerMenu.FilterSwitch.MATCH_TAG,
								!filter.matchTag
							)
						)

						disableTicks = 20
					}
				}
			}

		val blacklist =
			object : ButtonPanel(this@DriveViewerScreen, dock_left, 0f, 0f, 90f, 20f, TranslatableComponent("otm.filter.blacklist", no)) {
				override fun tick() {
					super.tick()

					val filter = menu.getFilter()
					if (filter != null) {
						widget?.message = TranslatableComponent("otm.filter.blacklist", if (filter.isBlacklist) yes else no)
						widget?.active = !isWidgetDisabled
					} else {
						widget?.active = false
					}
				}

				override fun onPress() {
					super.onPress()
					val filter = menu.getFilter()
					if (filter != null) {
						MatteryNetworking.send(
							null,
							FilterSwitchPacket(
								menu.containerId,
								DriveViewerMenu.FilterSwitch.BLACKLIST,
								!filter.isBlacklist
							)
						)

						disableTicks = 20
					}
				}
			}

		matchNbt.setDockMargin(0f, 4f, 0f, 0f)
		matchTag.setDockMargin(0f, 4f, 0f, 0f)
		blacklist.setDockMargin(0f, 4f, 0f, 0f)

		for (panel in settings) {
			panel.visible = false
		}

		return frame
	}

	companion object {
		const val FRAME_WIDTH = 210f
		const val FRAME_HEIGHT = 110f
		const val GRID_WIDTH = 9
		const val GRID_HEIGHT = 5
	}
}