package ru.dbotthepony.mc.otm.client.screen.panels

import com.mojang.blaze3d.systems.RenderSystem
import com.mojang.blaze3d.vertex.PoseStack
import net.minecraft.client.gui.components.AbstractWidget
import org.lwjgl.opengl.GL11
import ru.dbotthepony.mc.otm.client.screen.MatteryScreen

abstract class MinecraftWidgetPanel<T : AbstractWidget>(
	screen: MatteryScreen<*>,
	parent: EditablePanel?,
	x: Float,
	y: Float,
	width: Float,
	height: Float
) : EditablePanel(screen, parent, x, y, width, height) {
	var widget: T? = null
		protected set

	var disableTicks = 0
		set(value) {
			field = value
			widget?.active = value > 0
		}

	val isWidgetDisabled: Boolean
		get() = disableTicks > 0

	override fun tick() {
		super.tick()

		if (disableTicks > 0) {
			disableTicks--
		}
	}

	private fun recreate() {
		if (widget == null) {
			return
		}

		val instance = makeNew()
		instance.visible = visible
		instance.x = absoluteX.toInt()
		instance.y = absoluteY.toInt()

		if (disableTicks > 0) {
			instance.active = false
		}

		val widget = widget
		if (widget != null)
			copyValues(instance, widget)

		this.widget = instance
		configureNew(instance, true)
	}

	override fun widthUpdated(new: Float, old: Float) {
		recreate()
	}

	override fun heightUpdated(new: Float, old: Float) {
		recreate()
	}

	override fun visibilityChanges(new: Boolean, old: Boolean) {
		widget?.visible = new
	}

	protected abstract fun copyValues(new_widget: T, old_widget: T)
	protected abstract fun makeNew(): T
	protected abstract fun configureNew(widget: T, recreation: Boolean)

	fun getOrCreateWidget(): T {
		val widget = this.widget
		if (widget != null) return widget

		val instance = makeNew()
		instance.visible = visible

		if (disableTicks > 0) {
			instance.active = false
		}

		configureNew(instance, false)

		this.widget = instance
		return instance
	}

	override fun innerRender(stack: PoseStack, mouse_x: Float, mouse_y: Float, flag: Float) {
		RenderSystem.depthFunc(GL11.GL_ALWAYS)
		getOrCreateWidget().render(stack, mouse_x.toInt(), mouse_y.toInt(), flag)
		RenderSystem.depthFunc(GL11.GL_ALWAYS)
	}

	override fun charTypedInternal(p_94732_: Char, p_94733_: Int) = getOrCreateWidget().charTyped(p_94732_, p_94733_)
	override fun keyPressedInternal(p_94745_: Int, p_94746_: Int, p_94747_: Int) = getOrCreateWidget().keyPressed(p_94745_, p_94746_, p_94747_)
	override fun keyReleasedInternal(p_94750_: Int, p_94751_: Int, p_94752_: Int) = getOrCreateWidget().keyReleased(p_94750_, p_94751_, p_94752_)

	override fun mouseClickedInner(mouse_x: Double, mouse_y: Double, mouse_click_type: Int): Boolean {
		val (x1, y1) = screenToLocal(mouse_x, mouse_y)
		return getOrCreateWidget().mouseClicked(x1.toDouble(), y1.toDouble(), mouse_click_type)
	}

	override fun mouseReleasedInner(mouse_x: Double, mouse_y: Double, flag: Int): Boolean {
		val (x1, y1) = screenToLocal(mouse_x, mouse_y)
		return getOrCreateWidget().mouseReleased(x1.toDouble(), y1.toDouble(), flag)
	}
}