package ru.dbotthepony.mc.otm.client.screen.panels

import com.mojang.blaze3d.vertex.PoseStack
import ru.dbotthepony.mc.otm.client.render.*
import ru.dbotthepony.mc.otm.client.screen.MatteryScreen
import java.util.function.Consumer
import java.util.function.Supplier
import kotlin.math.floor

open class ScrollBarPanel(screen: MatteryScreen<*>, parent: EditablePanel?, x: Float, y: Float, height: Float) :
	EditablePanel(screen, parent, x, y, WIDTH, height) {

	open inner class ScrollBarButtonPanel : EditablePanel(screen, this@ScrollBarPanel, 1f, 1f, 12f, 15f) {
		var isScrolling = false
			protected set

		override fun innerRender(stack: PoseStack, mouse_x: Float, mouse_y: Float, flag: Float) {
			if (isScrolling) {
				scroll_bar_button_press.render(stack, 0f, 0f)
			} else if (isHovered) {
				scroll_bar_button_hover.render(stack, 0f, 0f)
			} else {
				scroll_bar_button.render(stack, 0f, 0f)
			}
		}

		protected var lastMouseY = 0.0

		override fun mouseClickedInner(mouse_x: Double, mouse_y: Double, flag: Int): Boolean {
			isScrolling = true
			ignoreMouseEventBoundaries = true
			lastMouseY = mouse_y
			return true
		}

		override fun mouseReleasedInner(mouse_x: Double, mouse_y: Double, flag: Int): Boolean {
			if (isScrolling) {
				isScrolling = false
				ignoreMouseEventBoundaries = false
				return true
			}

			return false
		}

		override fun mouseScrolledInner(mouse_x: Double, mouse_y: Double, scroll: Double): Boolean {
			return this@ScrollBarPanel.mouseScrolledInner(mouse_x, mouse_y, scroll)
		}

		override fun mouseDraggedInner(
			mouse_x: Double,
			mouse_y: Double,
			flag: Int,
			drag_x: Double,
			drag_y: Double
		): Boolean {
			if (isScrolling) {
				val diff = mouse_y - lastMouseY
				lastMouseY = mouse_y
				this@ScrollBarPanel.scroll = (scroll + diff / this@ScrollBarPanel.height).toFloat()
				updateScrollBarPosition()
				return true
			}

			return false
		}
	}

	val scrollButton = ScrollBarButtonPanel()
	var scrollCallback: Consumer<Float>? = null

	var scrollMultiplier = Supplier { 0.1f }

	fun setupRowMultiplier(row_supplier: Supplier<Int>) {
		scrollMultiplier = Supplier {
			val rows = row_supplier.get()

			if (rows < 1) {
				return@Supplier 0f
			}

			return@Supplier 1f / rows
		}
	}

	var scroll = 0f
		get() {
			return if (field < 0f) 0f else if (field > 1f) 1f else field
		}
		set(scroll) {
			val previous = field
			field = if (scroll < 0f) 0f else if (scroll > 1f) 1f else scroll

			if (previous != field) {
				onScrolled(field, previous)
			}
		}

	protected open fun onScrolled(new_scroll: Float, old_scroll: Float) {
		scrollCallback?.accept(new_scroll)
	}

	fun updateScrollBarPosition() {
		scrollButton.y = 1 + (height - 2 - scrollButton.height) * scroll
	}

	override fun performLayout() {
		super.performLayout()
		updateScrollBarPosition()
	}

	fun translateScrollValue(new_max: Int, old_max: Int) {
		if (new_max < 1 || old_max < 1) {
			return
		}

		scroll = scroll * old_max / new_max
	}

	fun getScroll(max_rows: Int): Int {
		if (max_rows <= 1)
			return 0

		return floor(scroll * max_rows + 0.5).toInt()
	}

	// public is for emulating input from outside
	public override fun mouseScrolledInner(mouse_x: Double, mouse_y: Double, scroll: Double): Boolean {
		this.scroll = (this.scroll - scroll * scrollMultiplier.get()).toFloat()
		updateScrollBarPosition()
		return true
	}

	override fun innerRender(stack: PoseStack, mouse_x: Float, mouse_y: Float, flag: Float) {
		scroll_bar_body.render(stack, y = 2f, height = height - 4)
		scroll_bar_top.render(stack)
		scroll_bar_bottom.render(stack, y = height - 2)
	}

	companion object {
		const val WIDTH = 14f
	}
}