
@file:Suppress("unused")

package ru.dbotthepony.mc.otm.core

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.core.Vec3i
import java.math.BigDecimal

operator fun BlockPos.plus(direction: Vec3i): BlockPos = this.offset(direction)
operator fun BlockPos.plus(direction: Direction): BlockPos = this.offset(direction.normal)
operator fun BlockPos.minus(direction: Vec3i): BlockPos = this.subtract(direction)
operator fun BlockPos.minus(direction: Direction): BlockPos = this.subtract(direction.normal)

operator fun Vec3i.plus(direction: Vec3i): Vec3i = this.offset(direction)
operator fun Vec3i.plus(direction: Direction): Vec3i = this.offset(direction.normal)
operator fun Vec3i.minus(direction: Vec3i): Vec3i = this.subtract(direction)
operator fun Vec3i.minus(direction: Direction): Vec3i = this.subtract(direction.normal)

operator fun Vec3i.times(int: Int): Vec3i = this.multiply(int)
operator fun Vec3i.times(double: Double): Vector = Vector(x * double, y * double, z * double)

fun BlockPos.asVector(): Vector {
	return Vector(x + 0.5, y + 0.5, z + 0.5)
}

/**
 * Performs type check+cast and sums two numbers.
 *
 * Returns the same type as on input (adding Double to Float will not return Double, etc.)
 */
fun Number.dynPlus(other: Number): Number {
	return when (this) {
		is Float -> this 	+ other.toFloat()
		is Double -> this 	+ other.toDouble()
		is Int -> this 		+ other.toInt()
		is Long -> this 	+ other.toLong()
		is Short -> this 	+ other.toShort()
		is Byte -> this 	+ other.toByte()

		is ImpreciseFraction -> {
			when (other) {
				is Float -> this 					+ other.toFloat()
				is Double -> this 					+ other.toDouble()
				is Int, is Byte, is Short -> this 	+ other.toInt()
				is Long -> this 					+ other.toLong()
				is ImpreciseFraction -> this 		+ other
				is Fraction -> this 				+ other.toImpreciseFraction()
				else -> throw IllegalStateException("I don't know what to do with ${other::class.qualifiedName} (right hand operand)")
			}
		}

		is Fraction -> {
			when (other) {
				is Float -> this 						+ other.toFloat()
				is Double -> this 						+ other.toDouble()
				is Int, is Byte, is Short -> this 		+ other.toInt()
				is Long -> this 						+ other.toLong()
				is ImpreciseFraction -> this 			+ other.toFraction()
				is Fraction -> this 					+ other
				else -> throw IllegalStateException("I don't know what to do with ${other::class.qualifiedName} (right hand operand)")
			}
		}

		else -> throw IllegalStateException("I don't know what to do with ${this::class.qualifiedName} (left hand operand)")
	}
}

/**
 * Performs type check+cast and subtracts two numbers.
 *
 * Returns the same type as on input (adding Double to Float will not return Double, etc.)
 */
fun Number.dynMinus(other: Number): Number {
	return when (this) {
		is Float -> this 	- other.toFloat()
		is Double -> this 	- other.toDouble()
		is Int -> this 		- other.toInt()
		is Long -> this 	- other.toLong()
		is Short -> this 	- other.toShort()
		is Byte -> this 	- other.toByte()

		is ImpreciseFraction -> {
			when (other) {
				is Float -> this 					- other.toFloat()
				is Double -> this 					- other.toDouble()
				is Int, is Byte, is Short -> this 	- other.toInt()
				is Long -> this 					- other.toLong()
				is ImpreciseFraction -> this 		- other
				is Fraction -> this 				- other.toImpreciseFraction()
				else -> throw IllegalStateException("I don't know what to do with ${other::class.qualifiedName} (right hand operand)")
			}
		}

		is Fraction -> {
			when (other) {
				is Float -> this 						- other.toFloat()
				is Double -> this 						- other.toDouble()
				is Int, is Byte, is Short -> this 		- other.toInt()
				is Long -> this 						- other.toLong()
				is ImpreciseFraction -> this 			- other.toFraction()
				is Fraction -> this 					- other
				else -> throw IllegalStateException("I don't know what to do with ${other::class.qualifiedName} (right hand operand)")
			}
		}

		else -> throw IllegalStateException("I don't know what to do with ${this::class.qualifiedName} (left hand operand)")
	}
}

/**
 * Performs type check+cast and multiplies two numbers.
 *
 * Returns the same type as on input (adding Double to Float will not return Double, etc.)
 */
fun Number.dynTimes(other: Number): Number {
	return when (this) {
		is Float -> this 	* other.toFloat()
		is Double -> this 	* other.toDouble()
		is Int -> this 		* other.toInt()
		is Long -> this 	* other.toLong()
		is Short -> this 	* other.toShort()
		is Byte -> this 	* other.toByte()

		is ImpreciseFraction -> {
			when (other) {
				is Float -> this 					* other.toFloat()
				is Double -> this 					* other.toDouble()
				is Int, is Byte, is Short -> this 	* other.toInt()
				is Long -> this 					* other.toLong()
				is ImpreciseFraction -> this 		* other
				is Fraction -> this 				* other.toImpreciseFraction()
				else -> throw IllegalStateException("I don't know what to do with ${other::class.qualifiedName} (right hand operand)")
			}
		}

		is Fraction -> {
			when (other) {
				is Float -> this 						* other.toFloat()
				is Double -> this 						* other.toDouble()
				is Int, is Byte, is Short -> this 		* other.toInt()
				is Long -> this 						* other.toLong()
				is ImpreciseFraction -> this 			* other.toFraction()
				is Fraction -> this 					* other
				else -> throw IllegalStateException("I don't know what to do with ${other::class.qualifiedName} (right hand operand)")
			}
		}

		else -> throw IllegalStateException("I don't know what to do with ${this::class.qualifiedName} (left hand operand)")
	}
}

/**
 * Performs type check+cast and divides two numbers.
 *
 * Returns the same type as on input (adding Double to Float will not return Double, etc.)
 */
fun Number.dynDiv(other: Number): Number {
	return when (this) {
		is Float -> this 	/ other.toFloat()
		is Double -> this 	/ other.toDouble()
		is Int -> this 		/ other.toInt()
		is Long -> this 	/ other.toLong()
		is Short -> this 	/ other.toShort()
		is Byte -> this 	/ other.toByte()

		is ImpreciseFraction -> {
			when (other) {
				is Float -> this 					/ other.toFloat()
				is Double -> this 					/ other.toDouble()
				is Int, is Byte, is Short -> this 	/ other.toInt()
				is Long -> this 					/ other.toLong()
				is ImpreciseFraction -> this 		/ other
				is Fraction -> this 				/ other.toImpreciseFraction()
				else -> throw IllegalStateException("I don't know what to do with ${other::class.qualifiedName} (right hand operand)")
			}
		}

		is Fraction -> {
			when (other) {
				is Float -> this 						/ other.toFloat()
				is Double -> this 						/ other.toDouble()
				is Int, is Byte, is Short -> this 		/ other.toInt()
				is Long -> this 						/ other.toLong()
				is ImpreciseFraction -> this 			/ other.toFraction()
				is Fraction -> this 					/ other
				else -> throw IllegalStateException("I don't know what to do with ${other::class.qualifiedName} (right hand operand)")
			}
		}

		else -> throw IllegalStateException("I don't know what to do with ${this::class.qualifiedName} (left hand operand)")
	}
}

val Number.isFractional get() = this is Float || this is Double || this is ImpreciseFraction || this is Fraction
val Number.isWhole get() = !isFractional

fun Number.toImpreciseFraction(): ImpreciseFraction {
	return when (this) {
		is ImpreciseFraction -> this
		is Float -> ImpreciseFraction(this)
		is Double -> ImpreciseFraction(this)
		is Int -> ImpreciseFraction(this)
		is Byte -> ImpreciseFraction(this)
		is Short -> ImpreciseFraction(this)
		is Long -> ImpreciseFraction(this)
		is Fraction -> this.toImpreciseFraction()
		else -> throw ClassCastException("Can not turn $this into ImpreciseFraction")
	}
}
