package ru.dbotthepony.mc.otm.core

import com.mojang.blaze3d.systems.RenderSystem
import ru.dbotthepony.mc.otm.client.render.RenderHelper

data class RGBAColor(val red: Float, val green: Float, val blue: Float, val alpha: Float = 1f) {
	constructor(r: Int, g: Int, b: Int) : this((r / 255f), (g / 255f), (b / 255f), 1f)
	constructor(r: Int, g: Int, b: Int, a: Int) : this((r / 255f), (g / 255f), (b / 255f), (a / 255f))

	constructor(color: Long) : this(
		(((color and -0x1000000) ushr 24) / 255f),
		(((color and 0xFF0000) ushr 16) / 255f),
		(((color and 0xFF00) ushr 8) / 255f),
		(((color and 0xFF)) / 255f)
	)

	fun toInt(): Int {
		val r = (this.red * 255).toInt()
		val g = (this.green * 255).toInt()
		val b = (this.blue * 255).toInt()
		return (r shl 16) or (g shl 8) or b
	}

	fun toIntInv(): Int {
		val r = (this.red * 255).toInt()
		val g = (this.green * 255).toInt()
		val b = (this.blue * 255).toInt()
		return (b shl 16) or (g shl 8) or r
	}

	fun setSystemColor() {
		setShaderColor()
		setDrawColor()
	}

	fun setShaderColor() {
		RenderSystem.setShaderColor(red, green, blue, alpha)
	}

	fun setDrawColor() {
		RenderHelper.setDrawColor(this)
	}

	companion object {
		val BLACK = RGBAColor(0f, 0f, 0f, 1f)
		val WHITE = RGBAColor(1f, 1f, 1f, 1f)
		val SLATE_GRAY = RGBAColor(64, 64, 64)
		val GRAY = RGBAColor(0x2C2C2CFFL)

		fun inv(color: Int): RGBAColor {
			val r = (color and -0x1000000 ushr 24) / 255f
			val g = (color and 0xFF0000 ushr 16) / 255f
			val b = (color and 0xFF00 ushr 8) / 255f
			val a = (color and 0xFF) / 255f
			return RGBAColor(a, b, g, r)
		}
	}
}