package ru.dbotthepony.mc.otm.graph

import net.minecraft.core.Direction
import ru.dbotthepony.mc.otm.IConditionalTickable
import ru.dbotthepony.mc.otm.ITickable

interface GraphNodeListener {
	fun onNeighbourTop(node: Graph6Node<*>) 		= onNeighbour(node, Direction.UP)
	fun onNeighbourBottom(node: Graph6Node<*>) 		= onNeighbour(node, Direction.DOWN)
	fun onNeighbourLeft(node: Graph6Node<*>) 		= onNeighbour(node, Direction.WEST)
	fun onNeighbourRight(node: Graph6Node<*>) 		= onNeighbour(node, Direction.EAST)
	fun onNeighbourFront(node: Graph6Node<*>) 		= onNeighbour(node, Direction.SOUTH)
	fun onNeighbourBack(node: Graph6Node<*>) 		= onNeighbour(node, Direction.NORTH)

	fun onNeighbour(node: Graph6Node<*>, direction: Direction)

	fun onUnNeighbourTop(node: Graph6Node<*>) 		= onUnNeighbour(node, Direction.UP)
	fun onUnNeighbourBottom(node: Graph6Node<*>) 	= onUnNeighbour(node, Direction.DOWN)
	fun onUnNeighbourLeft(node: Graph6Node<*>) 		= onUnNeighbour(node, Direction.WEST)
	fun onUnNeighbourRight(node: Graph6Node<*>) 	= onUnNeighbour(node, Direction.EAST)
	fun onUnNeighbourFront(node: Graph6Node<*>) 	= onUnNeighbour(node, Direction.SOUTH)
	fun onUnNeighbourBack(node: Graph6Node<*>) 		= onUnNeighbour(node, Direction.NORTH)

	fun onUnNeighbour(node: Graph6Node<*>, direction: Direction)
}

// Вершина графа, содержит то, к какому графу принадлежит, соседей и своё "значение"
class Graph6Node<T>(@JvmField val value: T, graph: Abstract6Graph<T>? = null) : IConditionalTickable {
	var graph: Abstract6Graph<T>? = null

	init {
		this.graph = graph
	}

	var top: Graph6Node<T>? = null
		set(value) {
			if (value != null && (graph == null || value.graph !== graph)) throw IllegalStateException("Can not be neighbour with node from different graph")

			if (field != value && this.value is GraphNodeListener) {
				if (field != null) {
					this.value.onUnNeighbourTop(field!!)
				}

				if (value != null) {
					this.value.onNeighbourTop(value)
				}
			}

			field = value
		}

	var bottom: Graph6Node<T>? = null
		set(value) {
			if (value != null && (graph == null || value.graph !== graph)) throw IllegalStateException("Can not be neighbour with node from different graph")

			if (field != value && this.value is GraphNodeListener) {
				if (field != null) {
					this.value.onUnNeighbourBottom(field!!)
				}

				if (value != null) {
					this.value.onNeighbourBottom(value)
				}
			}

			field = value
		}

	var left: Graph6Node<T>? = null
		set(value) {
			if (value != null && (graph == null || value.graph !== graph)) throw IllegalStateException("Can not be neighbour with node from different graph")

			if (field != value && this.value is GraphNodeListener) {
				if (field != null) {
					this.value.onUnNeighbourLeft(field!!)
				}

				if (value != null) {
					this.value.onNeighbourLeft(value)
				}
			}

			field = value
		}

	var right: Graph6Node<T>? = null
		set(value) {
			if (value != null && (graph == null || value.graph !== graph)) throw IllegalStateException("Can not be neighbour with node from different graph")

			if (field != value && this.value is GraphNodeListener) {
				if (field != null) {
					this.value.onUnNeighbourRight(field!!)
				}

				if (value != null) {
					this.value.onNeighbourRight(value)
				}
			}

			field = value
		}

	var front: Graph6Node<T>? = null
		set(value) {
			if (value != null && (graph == null || value.graph !== graph)) throw IllegalStateException("Can not be neighbour with node from different graph")

			if (field != value && this.value is GraphNodeListener) {
				if (field != null) {
					this.value.onUnNeighbourFront(field!!)
				}

				if (value != null) {
					this.value.onNeighbourFront(value)
				}
			}

			field = value
		}

	var back: Graph6Node<T>? = null
		set(value) {
			if (value != null && (graph == null || value.graph !== graph)) throw IllegalStateException("Can not be neighbour with node from different graph")

			if (field != value && this.value is GraphNodeListener) {
				if (field != null) {
					this.value.onUnNeighbourBack(field!!)
				}

				if (value != null) {
					this.value.onNeighbourBack(value)
				}
			}

			field = value
		}

	fun nullifyConnections() {
		top?.bottom = null
		bottom?.top = null
		left?.right = null
		right?.left = null
		front?.back = null
		back?.front = null

		top = null
		bottom = null
		left = null
		right = null
		front = null
		back = null
	}

	fun setToNeightbour(node: Graph6Node<T>, direction: Direction) {
		when (direction) {
			Direction.DOWN -> {
				node.top = this
				bottom = node
			}

			Direction.UP -> {
				node.bottom = this
				top = node
			}

			Direction.NORTH -> {
				node.front = this
				back = node
			}

			Direction.SOUTH -> {
				node.back = this
				front = node
			}

			Direction.WEST -> {
				node.right = this
				left = node
			}

			Direction.EAST -> {
				node.left = this
				right = node
			}
		}
	}

	override val canTick: Boolean
		get() {
			return if (value is IConditionalTickable) {
				value.canTick
			} else {
				value is ITickable
			}
		}

	override fun tick() {
		if (value is ITickable) {
			value.tick()
		} else {
			throw ClassCastException("$value is not tickable")
		}
	}

	var seen: Int = 0

	private fun _flood(): List<GraphFlooder<T>> {
		val list = ArrayList<GraphFlooder<T>>()
		var seen = Int.MAX_VALUE

		GraphFlooder.floodIf(top, seen) {
			seen = it.seen
			list.add(it)
		}

		GraphFlooder.floodIf(bottom, seen) {
			seen = it.seen
			list.add(it)
		}

		GraphFlooder.floodIf(left, seen) {
			seen = it.seen
			list.add(it)
		}

		GraphFlooder.floodIf(right, seen) {
			seen = it.seen
			list.add(it)
		}

		GraphFlooder.floodIf(front, seen) {
			seen = it.seen
			list.add(it)
		}

		GraphFlooder.floodIf(back, seen) {
			seen = it.seen
			list.add(it)
		}

		return list
	}

	fun flood(): List<GraphFlooder<T>> {
		top?.bottom = null
		bottom?.top = null
		left?.right = null
		right?.left = null
		front?.back = null
		back?.front = null

		val list = _flood()

		top?.bottom = this
		bottom?.top = this
		left?.right = this
		right?.left = this
		front?.back = this
		back?.front = this

		return list
	}

	var valid = true
		private set

	fun destroy(factory: () -> Abstract6Graph<T>): List<GraphFlooder<T>> {
		if (!valid) return emptyList()

		top?.bottom = null
		bottom?.top = null
		left?.right = null
		right?.left = null
		front?.back = null
		back?.front = null

		graph?.removeNode(this)

		var num = 0

		if (top != null) num++
		if (bottom != null) num++
		if (left != null) num++
		if (right != null) num++
		if (front != null) num++
		if (back != null) num++

		if (num < 2) {
			return emptyList()
		}

		val paths = _flood()

		if (paths.size < 2) {
			return paths
		}

		var biggest = paths[0]

		for (i in 1 until paths.size) {
			if (biggest.size() < paths[i].size()) {
				biggest = paths[i]
			}
		}

		for (flooder in paths) {
			if (flooder == biggest) continue

			val graph = factory()

			for (node in flooder.nodes) {
				node.graph?.removeNode(node)
				graph.addNode(node)
			}
		}

		return paths
	}
}
