package ru.dbotthepony.mc.otm.item

import ru.dbotthepony.mc.otm.capability.drive.DrivePool.isLegalAccess
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import net.minecraft.world.item.ItemStack
import net.minecraftforge.common.capabilities.ICapabilityProvider
import net.minecraftforge.common.util.LazyOptional
import ru.dbotthepony.mc.otm.capability.drive.IMatteryDrive
import ru.dbotthepony.mc.otm.capability.drive.ItemMatteryDrive
import net.minecraft.nbt.CompoundTag
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import net.minecraft.world.item.TooltipFlag
import net.minecraft.ChatFormatting
import net.minecraft.core.Direction
import net.minecraft.nbt.ListTag
import net.minecraft.nbt.Tag
import net.minecraft.network.chat.Component
import net.minecraft.network.chat.TextComponent
import net.minecraft.stats.Stats
import net.minecraft.world.item.Item
import net.minecraft.world.level.Level
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.event.ForgeEventFactory
import net.minecraftforge.eventbus.api.SubscribeEvent
import net.minecraftforge.event.entity.player.EntityItemPickupEvent
import ru.dbotthepony.mc.otm.capability.drive.DrivePool
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.set
import java.util.*

class PortableCondensationDriveItem(capacity: Int) :
	Item(Properties().stacksTo(1).tab(OverdriveThatMatters.INSTANCE.CREATIVE_TAB)) {
	val capacity: ImpreciseFraction

	init {
		this.capacity = ImpreciseFraction(capacity)
	}

	private inner class DriveCapability(private val stack: ItemStack) : ICapabilityProvider {
		private var uuid: UUID? = null

		private val resolver = LazyOptional.of<IMatteryDrive<*>> {
			if (!isLegalAccess()) {
				return@of ItemMatteryDrive.DUMMY
			}

			val uuid = uuid

			DrivePool.get(uuid!!, { tag: CompoundTag? ->
				val drive = ItemMatteryDrive(capacity, uuid)
				drive.deserializeNBT(tag!!)
				drive
			}, { ItemMatteryDrive(capacity, uuid) })
		}

		override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
			if (cap === MatteryCapability.DRIVE) {
				if (uuid == null) {
					val array = stack.orCreateTag.getLongArray("uuid")

					if (array.size == 2) {
						uuid = UUID(array[0], array[1])
					} else {
						uuid = UUID.randomUUID()
						stack.orCreateTag.putLongArray("uuid", longArrayOf(uuid!!.mostSignificantBits, uuid!!.leastSignificantBits))
					}
				}

				return resolver.cast()
			}

			return LazyOptional.empty()
		}
	}

	override fun appendHoverText(
		stack: ItemStack,
		p_41422_: Level?,
		p_41423_: MutableList<Component>,
		p_41424_: TooltipFlag
	) {
		super.appendHoverText(stack, p_41422_, p_41423_, p_41424_)
		if (stack.tag != null) {
			val uuid = stack.tag!!.getLongArray("uuid")
			if (uuid.size == 2) {
				p_41423_.add(TextComponent(UUID(uuid[0], uuid[1]).toString()).withStyle(ChatFormatting.GRAY))
			}
		}
	}

	override fun initCapabilities(stack: ItemStack, nbt: CompoundTag?): ICapabilityProvider {
		return DriveCapability(stack)
	}

	class FilterSettings() {
		@JvmField
		val items = Array<ItemStack>(MAX_FILTERS) { ItemStack.EMPTY }

		@JvmField
		var matchNbt = false

		@JvmField
		var matchTag = false

		@JvmField
		var isBlacklist = false

		constructor(tag: CompoundTag) : this() {
			val list = tag.getList("filter", Tag.TAG_COMPOUND.toInt())
			var i = 0

			for (tag in list) {
				if (tag is CompoundTag) {
					items[i++] = ItemStack.of(tag)
				}
			}

			matchNbt = tag.getBoolean("match_nbt")
			matchTag = tag.getBoolean("match_tag")
			isBlacklist = tag.getBoolean("blacklist")
		}

		fun serializeNBT(compound: CompoundTag) {
			val list = ListTag()

			for (i in 0 until MAX_FILTERS) {
				list.add(items[i].serializeNBT())
			}

			compound["filter"] = list
			compound["match_nbt"] = matchNbt
			compound["match_tag"] = matchTag
			compound["blacklist"] = isBlacklist
		}

		fun serializeNBT(drive: ItemStack) {
			serializeNBT(drive.orCreateTag)
		}

		fun matches(stack: ItemStack): Boolean {
			if (isBlacklist) {
				for (item in items) {
					if (!matchNbt && ItemStack.isSame(item, stack)) {
						return false
					}

					if (matchTag) {
						val thisTags = item.tags
						val stackTatgs = stack.tags.toArray()

						for (tag1 in thisTags) {
							if (stackTatgs.contains(tag1)) {
								return false
							}
						}
					}

					if (matchNbt && ItemStack.isSameItemSameTags(item, stack)) {
						return false
					}
				}

				return true
			} else {
				for (item in items) {
					var same = ItemStack.isSame(item, stack)

					if (!same && matchTag) {
						val thisTags = item.tags
						val stackTatgs = stack.tags.toArray()

						for (tag1 in thisTags) {
							if (stackTatgs.contains(tag1)) {
								same = true
								break
							}
						}
					}

					if (matchNbt) {
						if (same && ItemStack.tagMatches(item, stack)) {
							return true
						}
					} else {
						if (same) {
							return true
						}
					}
				}

				return false
			}
		}

		companion object {
			const val MAX_FILTERS = 12
		}
	}

	fun getFilterSettings(drive: ItemStack): FilterSettings {
		return FilterSettings(drive.orCreateTag)
	}

	@Suppress("unused")
	companion object {
		@SubscribeEvent
		fun onPickupEvent(event: EntityItemPickupEvent) {
			if (event.item.owner != null && event.item.owner != event.player.uuid && event.item.age < 200) {
				return
			}

			var amount = event.item.item.count
			val item = event.item.item.item

			for (stack in event.player.inventory.items) {
				val drive = stack.item

				if (drive is PortableCondensationDriveItem) {
					var doBreak = false

					stack.getCapability(MatteryCapability.DRIVE).ifPresent {
						val filter = drive.getFilterSettings(stack)

						if (filter.matches(event.item.item)) {
							val copy = event.item.item.copy()
							val remaining = (it as ItemMatteryDrive).insertStack(event.item.item, false)

							if (remaining.count == event.item.item.count) {
								return@ifPresent
							}

							copy.count -= remaining.count
							event.isCanceled = true
							ForgeEventFactory.firePlayerItemPickupEvent(event.player, event.item, copy)

							if (remaining.count > 0) {
								event.item.item.count = remaining.count
								event.player.take(event.item, amount - remaining.count)

								event.player.awardStat(Stats.ITEM_PICKED_UP.get(item), amount - remaining.count)
								event.player.onItemPickup(event.item)

								amount = remaining.count
							} else {
								event.player.take(event.item, amount)

								event.player.awardStat(Stats.ITEM_PICKED_UP.get(item), amount)
								event.player.onItemPickup(event.item)

								event.item.discard()
								doBreak = true
							}
						}
					}

					if (doBreak) return
				}
			}
		}
	}
}