package ru.dbotthepony.mc.otm.menu

import net.minecraft.world.SimpleContainer
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.item.ItemStack
import ru.dbotthepony.mc.otm.block.entity.AndroidStationBlockEntity
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.android.IAndroidCapability
import ru.dbotthepony.mc.otm.registry.MMenus

private class AndroidStationContainer(val player: Player) : SimpleContainer(1) {
	var android: IAndroidCapability? = null

	init {
		player.getCapability(MatteryCapability.ANDROID).ifPresent {
			android = it
			setItem(0, it.batteryItemStack)
		}
	}
}

private class AndroidBatterySlot(container: AndroidStationContainer, index: Int) : BatterySlot(container, index) {
	val android: IAndroidCapability? = container.android

	override fun set(stack: ItemStack) {
		super.set(stack)
		android?.batteryItemStack = stack
	}
}

class AndroidStationMenu @JvmOverloads constructor(
	containerID: Int,
	inventory: Inventory,
	tile: AndroidStationBlockEntity? = null
) : MatteryPoweredMenu(MMenus.ANDROID_STATION, containerID, inventory, tile) {
	val androidBattery: MatterySlot = AndroidBatterySlot(AndroidStationContainer(inventory.player), 0)

	init {
		addSlot(androidBattery)
		addInventorySlots()
	}

	override fun getWorkingSlotStart(): Int {
		return androidBattery.index
	}

	override fun getWorkingSlotEnd(): Int {
		return androidBattery.index + 1
	}
}