package ru.dbotthepony.mc.otm.menu

import net.minecraft.world.SimpleContainer
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.item.ItemStack
import net.minecraftforge.common.ForgeHooks
import net.minecraftforge.energy.CapabilityEnergy
import ru.dbotthepony.mc.otm.block.entity.ChemicalGeneratorBlockEntity
import ru.dbotthepony.mc.otm.menu.data.IntDataContainer
import ru.dbotthepony.mc.otm.menu.widget.LevelGaugeWidget
import ru.dbotthepony.mc.otm.menu.widget.ProgressGaugeWidget
import ru.dbotthepony.mc.otm.registry.MMenus

class ChemicalGeneratorMenu @JvmOverloads constructor(id: Int, inv: Inventory, tile: ChemicalGeneratorBlockEntity? = null)
	: MatteryMenu(MMenus.CHEMICAL_GENERATOR, id, inv, tile) {

	val container = tile?.container ?: SimpleContainer(3)

	val fuelSlot = object : MatterySlot(container, ChemicalGeneratorBlockEntity.SLOT_INPUT) {
		override fun mayPlace(p_40231_: ItemStack): Boolean {
			return ForgeHooks.getBurnTime(p_40231_, null) > 0
		}
	}

	val residueSlot = object : MatterySlot(container, ChemicalGeneratorBlockEntity.SLOT_RESIDUE) {
		override fun mayPlace(p_40231_: ItemStack): Boolean {
			return false
		}
	}

	val batterySlot = object : MatterySlot(container, ChemicalGeneratorBlockEntity.SLOT_BATTERY) {
		override fun mayPlace(p_40231_: ItemStack): Boolean {
			return p_40231_.getCapability(CapabilityEnergy.ENERGY).isPresent
		}
	}

	val progress = ProgressGaugeWidget(this) { 1f - tile!!.workingTicks.toFloat() / tile.workingTicksTotal }
	val energy = LevelGaugeWidget(this, tile?.energy)
	val burnTime = IntDataContainer()

	init {
		addSlot(fuelSlot)
		addSlot(batterySlot)
		addSlot(residueSlot)
		addDataContainer(burnTime)
		addInventorySlots()
	}

	override fun broadcastChanges() {
		super.broadcastChanges()
		progress.updateServer()
		burnTime.value = (tile as ChemicalGeneratorBlockEntity).workingTicks
	}

	override fun getWorkingSlotStart(): Int {
		return 0
	}

	override fun getWorkingSlotEnd(): Int {
		return 3
	}
}
