package ru.dbotthepony.mc.otm.menu

import net.minecraft.core.Direction
import kotlin.jvm.JvmOverloads
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.level.block.Block
import ru.dbotthepony.mc.otm.block.EnergyCounterBlock
import ru.dbotthepony.mc.otm.block.entity.EnergyCounterBlockEntity
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.menu.data.EnumDataContainer
import ru.dbotthepony.mc.otm.menu.data.ImpreciseFractionDataContainer
import ru.dbotthepony.mc.otm.menu.widget.NumberPlayerInputWidget
import ru.dbotthepony.mc.otm.menu.widget.OneWayPlayerInputWidget
import ru.dbotthepony.mc.otm.registry.MMenus
import java.math.BigDecimal

class EnergyCounterMenu @JvmOverloads constructor(
	p_38852_: Int,
	inventory: Inventory,
	tile: EnergyCounterBlockEntity? = null
) : MatteryMenu(MMenus.ENERGY_COUNTER, p_38852_, inventory, tile) {
	val passed = ImpreciseFractionDataContainer()
	val average = ImpreciseFractionDataContainer()
	val last20Ticks = ImpreciseFractionDataContainer()
	val lastTick = ImpreciseFractionDataContainer()

	val switchDirection: OneWayPlayerInputWidget
	val inputDirection = EnumDataContainer(Direction::class.java)

	val maxIO = NumberPlayerInputWidget(this)

	// TODO: Graph and proper networking for it
	private var ticksPassed = 0

	init {
		if (tile == null) {
			switchDirection = OneWayPlayerInputWidget(this)
			maxIO.asClient()
		} else {
			switchDirection = OneWayPlayerInputWidget(this) {
				tile.level?.setBlock(tile.blockPos, tile.blockState.setValue(EnergyCounterBlock.INPUT_DIRECTION, tile.blockState.getValue(EnergyCounterBlock.INPUT_DIRECTION).opposite), Block.UPDATE_ALL)
			}

			maxIO.withSupplier {
				return@withSupplier if (tile.ioLimit == null) MINUS_ONE else tile.ioLimit!!.toBigDecmial()
			}.withConsumer {
				if (it.signum() < 0) {
					tile.ioLimit = null
				} else {
					tile.ioLimit = ImpreciseFraction(it)
				}
			}
		}

		addDataContainer(passed)
		addDataContainer(average)
		addDataContainer(last20Ticks)
		addDataContainer(lastTick)
		addDataSlots(inputDirection)
	}

	override fun broadcastChanges() {
		if (tile is EnergyCounterBlockEntity) {
			passed.value = tile.passed
			average.value = tile.calcAverage(20)
			lastTick.value = tile.lastTick

			if (ticksPassed == 0) {
				last20Ticks.value = tile.sumHistory(20)
			}

			ticksPassed = (ticksPassed + 1) % 20
			inputDirection.value = tile.blockState.getValue(EnergyCounterBlock.INPUT_DIRECTION)
		}

		super.broadcastChanges()
	}

	override fun getWorkingSlotStart(): Int {
		return 0
	}

	override fun getWorkingSlotEnd(): Int {
		return 0
	}

	companion object {
		private val MINUS_ONE = -BigDecimal.ONE
	}
}
