package ru.dbotthepony.mc.otm.menu.data

import net.minecraft.world.inventory.ContainerData
import java.util.*

class EnumDataContainer<T : Enum<T>>(val enum: Class<T>) : ContainerData {
	val buffer = intArrayOf(0)

	constructor(enum: Class<T>, default: T) : this(enum) {
		value = default
	}

	var value: T
		get() = enum.enumConstants[buffer[0]]
		set(value) {
			require(value::class.java == enum) { "Invalid enum provided (expected $enum, got ${value::class.java})" }
			buffer[0] = value.ordinal
		}

	override fun get(p_39284_: Int): Int {
		return buffer[p_39284_]
	}

	override fun set(p_39285_: Int, p_39286_: Int) {
		buffer[p_39285_] = p_39286_
	}

	override fun getCount(): Int {
		return 1
	}
}

class UUIDDataContainer : ComplexDataContainer<UUID>(16) {
	private var _value = UUID(0, 0)

	override var value: UUID
		get() = _value
		set(value) {
			buffer.position(0).putLong(value.mostSignificantBits).putLong(value.leastSignificantBits)
			_value = value
			dirty = true
		}

	override fun construct() {
		_value = UUID(buffer.position(0).long, buffer.long)
	}
}
