package ru.dbotthepony.mc.otm.menu.data

import net.minecraft.client.Minecraft
import net.minecraft.network.FriendlyByteBuf
import net.minecraftforge.network.NetworkEvent
import ru.dbotthepony.mc.otm.menu.MatteryMenu
import java.nio.ByteBuffer
import java.util.function.Supplier

class MultiByteDataContainerPacket(val slotID: Int, val offset: Int, val payload: ByteArray) {
	fun play(context: Supplier<NetworkEvent.Context>) {
		context.get().packetHandled = true
		(Minecraft.getInstance().player?.containerMenu as? MatteryMenu)?.getDataContainer(slotID)?.handle(this)
	}

	fun write(buff: FriendlyByteBuf) {
		buff.writeInt(slotID)
		buff.writeInt(offset)
		buff.writeByteArray(payload)
	}

	companion object {
		private val empty = ByteArray(0)
		fun empty(slotID: Int) = MultiByteDataContainerPacket(slotID, 0, empty)

		fun read(buff: FriendlyByteBuf): MultiByteDataContainerPacket {
			return MultiByteDataContainerPacket(buff.readInt(), buff.readInt(), buff.readByteArray())
		}
	}
}

open class MultiByteDataContainer(bytes: Int) {
	init {
		require(bytes >= 1) { "Buffer size of $bytes does not make any sense" }
	}

	protected val buffer: ByteBuffer = ByteBuffer.allocate(bytes)
	var slotID = 0
	var dirty = false

	open fun handle(packet: MultiByteDataContainerPacket) {
		if (packet.payload.isEmpty()) {
			buffer.array().fill(0)
			return
		}

		val backed = buffer.array()

		for (i in 0 until packet.offset) {
			backed[i] = bzero
		}

		for (i in packet.payload.indices) {
			backed[i + packet.offset] = packet.payload[i]
		}

		for (i in packet.payload.size + packet.offset until backed.size) {
			backed[i] = bzero
		}
	}

	open fun makePacket(): MultiByteDataContainerPacket {
		var firstNonZero = 0
		val backed = buffer.array()

		for (i in 0 until backed.size) {
			if (backed[i] != bzero) {
				firstNonZero = i
				break
			}
		}

		if (firstNonZero == 0 && backed[0] == bzero) {
			return MultiByteDataContainerPacket.empty(slotID)
		}

		var lastNonZero = firstNonZero

		for (i in firstNonZero + 1 until backed.size) {
			if (backed[i] != bzero) {
				lastNonZero = i
			}
		}

		val payload = ByteArray(lastNonZero - firstNonZero + 1)
		var pi = 0

		for (i in firstNonZero .. lastNonZero) {
			payload[pi++] = backed[i]
		}

		return MultiByteDataContainerPacket(slotID, firstNonZero, payload)
	}

	companion object {
		private const val bzero: Byte = 0
	}
}

abstract class ComplexDataContainer<T>(bytes: Int) : MultiByteDataContainer(bytes) {
	abstract var value: T
	protected abstract fun construct()

	override fun handle(packet: MultiByteDataContainerPacket) {
		super.handle(packet)
		construct()
	}
}
