package ru.dbotthepony.mc.otm.menu.widget

import net.minecraft.network.FriendlyByteBuf
import net.minecraftforge.network.NetworkEvent
import ru.dbotthepony.mc.otm.menu.MatteryMenu
import ru.dbotthepony.mc.otm.menu.data.BooleanDataContainer
import ru.dbotthepony.mc.otm.network.MatteryNetworking
import java.util.function.Supplier

class BooleanPlayerInputPacket(val id: Int, val value: Boolean) {
	fun play(context: Supplier<NetworkEvent.Context>) {
		context.get().packetHandled = true

		context.get().enqueueWork {
			((context.get().sender?.containerMenu as? MatteryMenu)?.getWidget(id) as? BooleanPlayerInputWidget)?.userInput(value)
		}
	}

	fun write(buff: FriendlyByteBuf) {
		buff.writeInt(id)
		buff.writeBoolean(value)
	}

	companion object {
		fun read(buff: FriendlyByteBuf): BooleanPlayerInputPacket {
			return BooleanPlayerInputPacket(buff.readInt(), buff.readBoolean())
		}
	}
}

class BooleanPlayerInputWidget(menu: MatteryMenu) : AbstractWidget(menu) {
	private val container = BooleanDataContainer()

	init {
		addDataSlots(container)
	}

	var supplier: (() -> Boolean)? = null
	var clicker: ((Boolean) -> Unit)? = null
	var value by container::value

	fun withSupplier(func: () -> Boolean): BooleanPlayerInputWidget {
		supplier = func
		return this
	}

	fun withClicker(func: (Boolean) -> Unit): BooleanPlayerInputWidget {
		clicker = func
		return this
	}

	fun asClient(): BooleanPlayerInputWidget {
		supplier = null
		clicker = {
			MatteryNetworking.CHANNEL.sendToServer(BooleanPlayerInputPacket(slotID, it))
		}

		return this
	}

	fun userInput(newValue: Boolean) {
		val clicker = clicker

		if (clicker != null) {
			clicker.invoke(newValue)
			updateServer()
		}
	}

	fun switchValue() {
		userInput(!value)
	}

	override fun updateServer() {
		val supplier = supplier

		if (supplier != null) {
			container.value = supplier.invoke()
		}
	}
}
