package ru.dbotthepony.mc.otm.menu.widget

import ru.dbotthepony.mc.otm.capability.IMatteryEnergyStorage
import ru.dbotthepony.mc.otm.capability.matter.IMatterHandler
import ru.dbotthepony.mc.otm.capability.matter.IPatternStorage
import ru.dbotthepony.mc.otm.core.Fraction
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.menu.MatteryMenu
import ru.dbotthepony.mc.otm.menu.data.FractionDataContainer
import ru.dbotthepony.mc.otm.menu.data.ImpreciseFractionDataContainer

@Suppress("unused")
class LevelGaugeWidget(menu: MatteryMenu) : AbstractWidget(menu) {
	var level = { ImpreciseFraction.ONE}
	var maxLevel = {ImpreciseFraction.ONE}

	val levelContainer = ImpreciseFractionDataContainer()
	val maxLevelContainer = ImpreciseFractionDataContainer()

	constructor(
		menu: MatteryMenu,
		power: IMatteryEnergyStorage?
	) : this(menu) {
		if (power == null) return

		this.level = power::batteryLevel
		this.maxLevel = power::maxBatteryLevel
	}

	constructor(
		menu: MatteryMenu,
		matter: IMatterHandler?
	) : this(menu) {
		if (matter == null) return

		this.level = matter::storedMatter
		this.maxLevel = matter::maxStoredMatter
	}

	constructor(
		menu: MatteryMenu,
		patterns: IPatternStorage?
	) : this(menu) {
		if (patterns == null) return

		this.level = {ImpreciseFraction(patterns.stored)}
		this.maxLevel = {ImpreciseFraction(patterns.capacity)}
	}

	constructor(
		menu: MatteryMenu,
		level: () -> ImpreciseFraction,
		maxLevel: () -> ImpreciseFraction,
	) : this(menu) {
		this.level = level
		this.maxLevel = maxLevel
	}

	init {
		addDataContainer(levelContainer)
		addDataContainer(maxLevelContainer)
	}

	override fun updateServer() {
		levelContainer.value = level.invoke()
		maxLevelContainer.value = maxLevel.invoke()
	}

	fun level(): ImpreciseFraction = levelContainer.value
	fun maxLevel(): ImpreciseFraction = maxLevelContainer.value
	fun percentage(): Float = levelContainer.value.percentage(maxLevelContainer.value)
}
