package ru.dbotthepony.mc.otm.menu.widget

import net.minecraft.network.FriendlyByteBuf
import net.minecraftforge.network.NetworkEvent
import ru.dbotthepony.mc.otm.menu.MatteryMenu
import ru.dbotthepony.mc.otm.menu.data.BigDecimalDataContainer
import ru.dbotthepony.mc.otm.network.MatteryNetworking
import ru.dbotthepony.mc.otm.network.NetworkHelper
import java.math.BigDecimal
import java.util.function.Supplier

class NumberPlayerInputPacket(val id: Int, val value: BigDecimal) {
	fun play(context: Supplier<NetworkEvent.Context>) {
		context.get().packetHandled = true

		context.get().enqueueWork {
			((context.get().sender?.containerMenu as? MatteryMenu)?.getWidget(id) as? NumberPlayerInputWidget)?.userInput(value)
		}
	}

	fun write(buff: FriendlyByteBuf) {
		buff.writeInt(id)
		NetworkHelper.writeDecimal(buff, value)
	}

	companion object {
		fun read(buff: FriendlyByteBuf): NumberPlayerInputPacket {
			return NumberPlayerInputPacket(buff.readInt(), NetworkHelper.readDecimal(buff))
		}
	}
}

class NumberPlayerInputWidget(menu: MatteryMenu) : AbstractWidget(menu) {
	private val container = BigDecimalDataContainer()
	val value by container::value

	var supplier: (() -> BigDecimal)? = null
	var consumer: ((BigDecimal) -> Unit)? = null
	var minValue: BigDecimal? = null
	var maxValue: BigDecimal? = null

	init {
		addDataContainer(container)
	}

	fun withSupplier(func: () -> BigDecimal): NumberPlayerInputWidget {
		supplier = func
		return this
	}

	fun withConsumer(func: (BigDecimal) -> Unit): NumberPlayerInputWidget {
		consumer = func
		return this
	}

	fun asClient(): NumberPlayerInputWidget {
		supplier = null

		consumer = {
			MatteryNetworking.CHANNEL.sendToServer(NumberPlayerInputPacket(slotID, it))
		}

		return this
	}

	fun userInput(newValue: BigDecimal) {
		val consumer = consumer

		if (consumer != null) {
			@Suppress("name_shadowing")
			var newValue = newValue

			if (minValue != null) {
				newValue = newValue.max(minValue!!)
			}

			if (maxValue != null) {
				newValue = newValue.min(maxValue!!)
			}

			consumer.invoke(newValue)
			updateServer()
		}
	}

	override fun updateServer() {
		val supplier = supplier

		if (supplier != null) {
			container.value = supplier.invoke()
		}
	}
}
