package ru.dbotthepony.mc.otm.storage

import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack
import net.minecraftforge.registries.ForgeRegistries
import net.minecraftforge.registries.ForgeRegistry
import ru.dbotthepony.mc.otm.core.ImpreciseFraction

/**
 * constructors always copy its input.
 */
class ItemStackWrapper : IStorageStack {
	/**
	 * [ItemStack] representing what item is this.
	 *
	 * In most cases you want to use [stack] instead.
	 */
	val item: ItemStack
	val registryName get() = item.item.registryName!!
	override var count: ImpreciseFraction
		set(value) { field = value.floor() }

	constructor(item: ItemStack) {
		this.item = item.copy()
		this.count = ImpreciseFraction(item.count)
		this.item.count = 1
		this.longHashCode = longHashCode(this.item.item)
	}

	constructor(item: ItemStackWrapper) {
		this.item = item.item.copy()
		this.count = item.count
		this.item.count = 1
		// this.longHashCode = longHashCode(this.item.item)
		this.longHashCode = item.longHashCode
	}

	override fun copy(): ItemStackWrapper {
		return ItemStackWrapper(this)
	}

	override val maxStackSize get() = ImpreciseFraction(item.maxStackSize)

	override val isEmpty: Boolean get() = item.isEmpty || !count.isPositive
	override val longHashCode: Long

	/**
	 * [ItemStack] with valid amount and which can be modified after
	 */
	val stack: ItemStack get() = item.copy().also { it.count = count.toInt() }

	override fun toString(): String {
		return "ItemStackWrapper[$count $registryName]"
	}

	override fun sameItem(other: IStorageStack): Boolean {
		if (this === other)
			return true

		if (other is ItemStackWrapper)
			return ItemStack.isSameItemSameTags(item, other.item)

		return false
	}

	fun sameItem(other: ItemStack): Boolean {
		return ItemStack.isSameItemSameTags(item, other)
	}

	companion object {
		@JvmField
		val EMPTY = ItemStackWrapper(ItemStack.EMPTY)

		fun longHashCode(item: Item) = (ForgeRegistries.ITEMS as? ForgeRegistry<Item>)?.getID(item)?.toLong() ?: System.identityHashCode(item).toLong()
	}
}
