package ru.dbotthepony.mc.otm.tests

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import ru.dbotthepony.mc.otm.core.Fraction
import ru.dbotthepony.mc.otm.menu.data.FractionDataContainer
import ru.dbotthepony.mc.otm.network.NetworkHelper
import kotlin.experimental.and

object NetworkingTests {
	@Test
	@DisplayName("Test FractionDataContainer store/load")
	fun fraction() {
		val container = FractionDataContainer()
		var frac = Fraction(18, 4)

		container.value = frac
		container.handle(container.makePacket())
		assert(container.value.compareTo(frac) == 0) { "${container.value} != $frac" }

		frac = Fraction(2_000_000, 1)

		container.value = frac
		container.handle(container.makePacket())
		assert(container.value.compareTo(frac) == 0) { "${container.value} != $frac" }
	}

	@Test
	@DisplayName("Test NetworkHelper byte/short transition")
	fun networkHelper() {
		var bytes = byteArrayOf(7, 8, -10)
		var shorts = NetworkHelper.bytesToShorts(bytes)

		assert(shorts.size == 2) { "Shorts array is not size of 2, but size of ${shorts.size}" }
		assert((shorts[0].toInt() and 0xFF).toByte() == bytes[0]) { "${shorts[0] and 0xFF} != ${bytes[0]} ${shorts.joinToString()}" }
		assert(((shorts[0].toInt() and 0xFF00) shr 8).toByte() == bytes[1]) { "${(shorts[0].toInt() and 0xFF00) shr 8} != ${bytes[1]} ${shorts.joinToString()}" }
		assert((shorts[1].toInt() and 0xFF).toByte() == bytes[2]) { "${shorts[1] and 0xFF} != ${bytes[2]} ${shorts.joinToString()}" }

		bytes = byteArrayOf(7, 8, -10, 4)
		shorts = NetworkHelper.bytesToShorts(bytes)

		assert(shorts.size == 2) { "Shorts array is not size of 2, but size of ${shorts.size}" }
		assert((shorts[0].toInt() and 0xFF).toByte() == bytes[0]) { "${shorts[0] and 0xFF} != ${bytes[0]} ${shorts.joinToString()}" }
		assert(((shorts[0].toInt() and 0xFF00) shr 8).toByte() == bytes[1]) { "${(shorts[0].toInt() and 0xFF00) shr 8} != ${bytes[1]} ${shorts.joinToString()}" }
		assert((shorts[1].toInt() and 0xFF).toByte() == bytes[2]) { "${shorts[1] and 0xFF} != ${bytes[2]} ${shorts.joinToString()}" }
		assert(((shorts[1].toInt() and 0xFF00) shr 8).toByte() == bytes[3]) { "${(shorts[1].toInt() and 0xFF00) shr 8} != ${bytes[3]} ${shorts.joinToString()}" }
	}
}
